@@ -22,6 +22,7 @@
 
 #include "basic.h"
 #include "ex.h"
+#include "expairseq.h"
 #include "ex_utils.h"
 #include "numeric.h"
 #include "power.h"
@@ -104,9 +105,6 @@ basic::basic(const archive_node &n, lst& /*unused*/) : flags(0)
 		throw (std::runtime_error("archive node contains no class name"));
 }
 
-/** Unarchive the object. */
-DEFAULT_UNARCHIVE(basic)
-
 /** Archive the object. */
 void basic::archive(archive_node &n) const
 {
@@ -140,40 +138,40 @@ void basic::print_dispatch(const registered_class_info & ri, const print_context
 	const registered_class_info * r_info = &ri;
 	const print_context_class_info * pc_info = &c.get_class_info();
 
-next_class:
-	const std::vector<print_functor> & pdt = r_info->options.get_print_dispatch_table();
-
-next_context:
+	const std::vector<print_functor> * pdt = &r_info->options.get_print_dispatch_table();
 	unsigned id = pc_info->options.get_id();
-	if (id >= pdt.size() || !(pdt[id].is_valid())) {
 
-		// Method not found, try parent print_context class
-		const print_context_class_info * parent_pc_info = pc_info->get_parent();
-		if (parent_pc_info != nullptr) {
-			pc_info = parent_pc_info;
-			goto next_context;
-		}
+        while(id >= pdt->size() or not ((*pdt)[id].is_valid())) {
 
-		// Method still not found, try parent class
-		const registered_class_info * parent_reg_info = r_info->get_parent();
-		if (parent_reg_info != nullptr) {
-			r_info = parent_reg_info;
-			pc_info = &c.get_class_info();
-			goto next_class;
-		}
+                // Method not found, try parent print_context class
+                const print_context_class_info * parent_pc_info = pc_info->get_parent();
+                if (parent_pc_info != nullptr) {
+                        pc_info = parent_pc_info;
+                        id = pc_info->options.get_id();
+                        continue;
+                }
 
-		// Method still not found. This shouldn't happen because basic (the
-		// base class of the algebraic hierarchy) registers a method for
-		// print_context (the base class of the print context hierarchy),
-		// so if we end up here, there's something wrong with the class
-		// registry.
-		throw (std::runtime_error(std::string("basic::print(): method for ") + class_name() + "/" + c.class_name() + " not found"));
+                // Method still not found, try parent class
+                const registered_class_info * parent_reg_info = r_info->get_parent();
+                if (parent_reg_info != nullptr) {
+                        r_info = parent_reg_info;
+                        pc_info = &c.get_class_info();
+                        pdt = &r_info->options.get_print_dispatch_table();
+                        id = pc_info->options.get_id();
+                        continue;
+                }
 
-	} else {
+                // Method still not found. This shouldn't happen because basic (the
+                // base class of the algebraic hierarchy) registers a method for
+                // print_context (the base class of the print context hierarchy),
+                // so if we end up here, there's something wrong with the class
+                // registry.
+                throw (std::runtime_error(std::string("basic::print(): method for ") + class_name() + "/" + c.class_name() + " not found"));
 
-		// Call method
-		pdt[id](*this, c, level);
-	}
+        }
+
+        // Call method
+        (*pdt)[id](*this, c, level);
 }
 
 /** Default output to stream. */
@@ -284,14 +282,19 @@ ex & basic::operator[](size_t i)
 	return let_op(i);
 }
 
+void basic::set_epseq_from(size_t i, ex e)
+{
+        dynamic_cast<expairseq&>(*this).set_pair_from(i, e);
+}
+
 /** Test for occurrence of a pattern.  An object 'has' a pattern if it matches
  *  the pattern itself or one of the children 'has' it.  As a consequence
  *  (according to the definition of children) given e=x+y+z, e.has(x) is true
  *  but e.has(x+y) is false. */
 bool basic::has(const ex & pattern, unsigned options) const
 {
-	lst repl_lst;
-	if (match(pattern, repl_lst))
+	exmap map;
+	if (match(pattern, map))
 		return true;
 	for (size_t i=0; i<nops(); i++)
 		if (op(i).has(pattern, options))
@@ -319,12 +322,12 @@ ex basic::map(map_function & f) const
 		}
 	}
 
-	if (copy != nullptr) {
-		copy->setflag(status_flags::dynallocated);
-		copy->clearflag(status_flags::hash_calculated | status_flags::expanded);
-		return *copy;
-	} else
-		return *this;
+	if (copy == nullptr)
+	        return *this;
+
+        copy->setflag(status_flags::dynallocated);
+        copy->clearflag(status_flags::hash_calculated | status_flags::expanded);
+	return *copy;
 }
 
 /** Check whether this is a polynomial in the given variables. */
@@ -336,13 +339,13 @@ bool basic::is_polynomial(const ex & var) const
 /** Return degree of highest power in object s. */
 numeric basic::degree(const ex & s) const
 {
-	return is_equal(ex_to<basic>(s)) ? 1 : 0;
+	return is_equal(ex_to<basic>(s)) ? *_num1_p : *_num0_p;
 }
 
 /** Return degree of lowest power in object s. */
 numeric basic::ldegree(const ex & s) const
 {
-	return is_equal(ex_to<basic>(s)) ? 1 : 0;
+	return is_equal(ex_to<basic>(s)) ? *_num1_p : *_num0_p;
 }
 
 /** Return coefficient of degree n in object s. */
@@ -350,8 +353,8 @@ ex basic::coeff(const ex & s, const ex & n) const
 {
 	if (is_equal(ex_to<basic>(s)))
 		return n.is_one() ? _ex1 : _ex0;
-	else
-		return n.is_zero() ? *this : _ex0;
+
+        return n.is_zero() ? *this : _ex0;
 }
 
 /** Sort expanded expression in terms of powers of some object(s).
@@ -367,9 +370,7 @@ ex basic::collect(const ex & s, bool distributed) const
 			return *this;
 		if (s.nops() == 1)
 			return collect(s.op(0));
-
-		else if (distributed) {
-
+		if (distributed) {
 			x = this->expand();
 			if (! is_exactly_a<add>(x))
 				return x; 
@@ -443,16 +444,13 @@ ex basic::evalf(int level, PyObject* parent) const
 {
 	if (nops() == 0)
 		return *this;
-	else {
-		if (level == 1)
-			return *this;
-		else if (level == -max_recursion_level)
-			throw(std::runtime_error("max recursion level reached"));
-		else {
-			evalf_map_function map_evalf(level - 1, parent);
-			return map(map_evalf);
-		}
-	}
+        if (level == 1)
+                return *this;
+        if (level == -max_recursion_level)
+                throw(std::runtime_error("max recursion level reached"));
+
+        evalf_map_function map_evalf(level - 1, parent);
+        return map(map_evalf);
 }
 
 /** Evaluate sums, products and integer powers of matrices. */
@@ -520,7 +518,7 @@ bool basic::contract_with(exvector::iterator /*unused*/, exvector::iterator /*un
 /** Check whether the expression matches a given pattern. For every wildcard
  *  object in the pattern, an expression of the form "wildcard == matching_expression"
  *  is added to repl_lst. */
-bool basic::match(const ex & pattern, lst & repl_lst) const
+bool basic::match(const ex & pattern, exmap& map) const
 {
 /*
 	Sweet sweet shapes, sweet sweet shapes,
@@ -543,40 +541,37 @@ bool basic::match(const ex & pattern, lst & repl_lst) const
 		// Wildcard matches anything, but check whether we already have found
 		// a match for that wildcard first (if so, the earlier match must be
 		// the same expression)
-		for (const auto & elem : repl_lst) {
-			if (elem.op(0).is_equal(pattern))
-				return is_equal(ex_to<basic>(elem.op(1)));
-		}
-		repl_lst.append(pattern == *this);
+                const auto& it = map.find(pattern);
+                if (it != map.end())
+		        return is_equal(ex_to<basic>(it->second));
+		map[pattern] = *this;
 		return true;
+	} 
 
-	} else {
+        // Expression must be of the same type as the pattern
+        if (tinfo() != ex_to<basic>(pattern).tinfo())
+                return false;
 
-		// Expression must be of the same type as the pattern
-		if (tinfo() != ex_to<basic>(pattern).tinfo())
-			return false;
+        // Number of subexpressions must match
+        if (nops() != pattern.nops())
+                return false;
 
-		// Number of subexpressions must match
-		if (nops() != pattern.nops())
-			return false;
+        // No subexpressions? Then just compare the objects (there can't be
+        // wildcards in the pattern)
+        if (nops() == 0)
+                return is_equal_same_type(ex_to<basic>(pattern));
 
-		// No subexpressions? Then just compare the objects (there can't be
-		// wildcards in the pattern)
-		if (nops() == 0)
-			return is_equal_same_type(ex_to<basic>(pattern));
+        // Check whether attributes that are not subexpressions match
+        if (!match_same_type(ex_to<basic>(pattern)))
+                return false;
 
-		// Check whether attributes that are not subexpressions match
-		if (!match_same_type(ex_to<basic>(pattern)))
-			return false;
+        // Otherwise the subexpressions must match one-to-one
+        for (size_t i=0; i<nops(); i++)
+                if (!op(i).match(pattern.sorted_op(i), map))
+                        return false;
 
-		// Otherwise the subexpressions must match one-to-one
-		for (size_t i=0; i<nops(); i++)
-			if (!op(i).match(pattern.sorted_op(i), repl_lst))
-				return false;
-
-		// Looks similar enough, match found
-		return true;
-	}
+        // Looks similar enough, match found
+        return true;
 }
 
 /** Helper function for subs(). Does not recurse into subexpressions. */
@@ -586,14 +581,23 @@ ex basic::subs_one_level(const exmap & m, unsigned options) const
 
         if (options & subs_options::no_pattern) {
                 ex thisex = *this;
+                if (is_exactly_a<numeric>(thisex))
+                        return ex_to<numeric>(thisex).subs(m, options);
                 for (const auto & pair : m)
                         if (thisex.is_equal(pair.first))
                                 return pair.second;
 	} else {
                 for (const auto & elem : m) {
-			lst repl_lst;
-			if (match(ex_to<basic>(elem.first), repl_lst))
-				return elem.second.subs(repl_lst, options | subs_options::no_pattern); // avoid infinite recursion when re-substituting the wildcards
+                        exmap map;
+			if (match(ex_to<basic>(elem.first), map)) {
+			        lst repl_lst;
+                                for (const auto& pair : map)
+                                        repl_lst.append(pair.first == pair.second);
+                                // avoid infinite recursion when
+                                // re-substituting the wildcards
+				return elem.second.subs(repl_lst,
+                                          options | subs_options::no_pattern);
+                        }
 		}
 	}
 
@@ -693,10 +697,9 @@ ex basic::derivative(const symbol & s) const
 {
 	if (nops() == 0)
 		return _ex0;
-	else {
-		derivative_map_function map_derivative(s);
-		return map(map_derivative);
-	}
+
+        derivative_map_function map_derivative(s);
+        return map(map_derivative);
 }
 
 /** Returns order relation between two objects of same type.  This needs to be
@@ -782,10 +785,10 @@ ex basic::expand(unsigned options) const
 {
 	if (nops() == 0)
 		return (options == 0) ? setflag(status_flags::expanded) : *this;
-	else {
-		expand_map_function map_expand(options);
-		return ex_to<basic>(map(map_expand)).setflag(options == 0 ? status_flags::expanded : 0);
-	}
+
+        expand_map_function map_expand(options);
+        return ex_to<basic>(map(map_expand)).setflag(options == 0 ? status_flags::expanded : 0);
+	
 }
 
 
@@ -811,33 +814,15 @@ int basic::compare(const basic & other) const
 	compare_statistics.compare_same_hashvalue++;
 #endif
 
-	const tinfo_t& typeid_this = tinfo();//typeid(*this);
-	const tinfo_t& typeid_other = other.tinfo();//typeid(other);
+	const tinfo_t& typeid_this = tinfo();
+	const tinfo_t& typeid_other = other.tinfo();
 	if (typeid_this == typeid_other) {
-//		int cmpval = compare_same_type(other);
-//		if (cmpval!=0) {
-//			std::cout << "hash collision, same type: " 
-//			          << *this << " and " << other << std::endl;
-//			this->print(print_tree(std::cout));
-//			std::cout << " and ";
-//			other.print(print_tree(std::cout));
-//			std::cout << std::endl;
-//		}
-//		return cmpval;
 #ifdef GINAC_COMPARE_STATISTICS
 		compare_statistics.compare_same_type++;
 #endif
 		return compare_same_type(other);
-	} else {
-//		std::cout << "hash collision, different types: " 
-//		          << *this << " and " << other << std::endl;
-//		this->print(print_tree(std::cout));
-//		std::cout << " and ";
-//	 	other.print(print_tree(std::cout));
-//	 	std::cout << std::endl;
-		//return (typeid_this.before(typeid_other) ? -1 : 1);
-		return (typeid_this<typeid_other ? -1 : 1);
-	}
+	} 
+        return (typeid_this<typeid_other ? -1 : 1);
 }
 
 /** Test for syntactic equality.
