@@ -4,6 +4,7 @@
 
 /*
  *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  (C) 2015-2018 Ralf Stephan <ralf@ark.in-berlin.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,12 +33,13 @@
 #include "utils.h"
 #include "infinity.h"
 #include "compiler.h"
+#include "cmatcher.h"
 
 #include <iostream>
 #include <algorithm>
+#include <numeric>
 #include <string>
 #include <stdexcept>
-#include <iterator>
 
 #if EXPAIRSEQ_USE_HASHTAB
 #include <cmath>
@@ -157,7 +159,7 @@ expairseq::expairseq(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst
 		ex lcoeff;
 		n.find_ex_by_loc(loc++, lrest, sym_lst);
 		n.find_ex_by_loc(loc++, lcoeff, sym_lst);
-		seq.push_back(expair(lrest, lcoeff));
+		seq.emplace_back(lrest, lcoeff);
 	}
 
         ex oc;
@@ -178,8 +180,6 @@ void expairseq::archive(archive_node &n) const
 	n.add_ex("overall_coeff", ex(overall_coeff));
 }
 
-DEFAULT_UNARCHIVE(expairseq)
-
 //////////
 // functions overriding virtual functions from base classes
 //////////
@@ -280,8 +280,7 @@ size_t expairseq::nops() const
 {
 	if (overall_coeff_equals_default())
 		return seq.size();
-	else
-		return seq.size()+1;
+        return seq.size()+1;
 }
 
 const ex expairseq::op(size_t i) const
@@ -311,16 +310,13 @@ ex expairseq::map(map_function &f) const
 
 	if (overall_coeff_equals_default())
 		return thisexpairseq(std::move(v), default_overall_coeff(), true);
-	else {
-		ex newcoeff = f(overall_coeff);
-		if(is_exactly_a<numeric>(newcoeff))
-			return thisexpairseq(std::move(v),
-                                        ex_to<numeric>(newcoeff), true);
-		else {
-			v->push_back(split_ex_to_pair(newcoeff));
-			return thisexpairseq(std::move(v), default_overall_coeff(), true);
-		}
-	}
+        ex newcoeff = f(overall_coeff);
+        if(is_exactly_a<numeric>(newcoeff))
+                return thisexpairseq(std::move(v),
+                                ex_to<numeric>(newcoeff), true);
+
+        v->push_back(split_ex_to_pair(newcoeff));
+        return thisexpairseq(std::move(v), default_overall_coeff(), true);
 }
 
 /** Perform coefficient-wise automatic term rewriting rules in this class. */
@@ -372,117 +368,6 @@ ex expairseq::conjugate() const
 	return result;
 }
 
-bool expairseq::match(const ex & pattern, lst & repl_lst) const
-{
-	// This differs from basic::match() because we want "a+b+c+d" to
-	// match "d+*+b" with "*" being "a+c", and we want to honor commutativity
-
-	if (this->tinfo() == ex_to<basic>(pattern).tinfo()) {
-
-		// Check whether global wildcard (one that matches the "rest of the
-		// expression", like "*" above) is present
-		bool has_global_wildcard = false;
-		ex global_wildcard;
-		for (size_t i=0; i<pattern.nops(); i++) {
-			if (is_exactly_a<wildcard>(pattern.sorted_op(i))) {
-				has_global_wildcard = true;
-				global_wildcard = pattern.sorted_op(i);
-				break;
-			}
-		}
-
-		// Even if the expression does not match the pattern, some of
-		// its subexpressions could match it. For example, x^5*y^(-1)
-		// does not match the pattern $0^5, but its subexpression x^5
-		// does. So, save repl_lst in order to not add bogus entries.
-		lst tmp_repl = repl_lst;
-
-		// Unfortunately, this is an O(N^2) operation because we can't
-		// sort the pattern in a useful way...
-
-		// Chop into terms
-		exvector ops;
-		ops.reserve(nops());
-		for (size_t i=0; i<nops(); i++)
-			ops.push_back(stable_op(i));
-
-		// Now, for every term of the pattern, look for a matching term in
-		// the expression and remove the match
-		for (size_t i=0; i<pattern.nops(); i++) {
-			ex p = ex_to<expairseq>(pattern).stable_op(i);
-			if (has_global_wildcard && p.is_equal(global_wildcard))
-				continue;
-			auto it = ops.begin(), itend = ops.end();
-			while (it != itend) {
-				auto last_el = repl_lst.end();
-				--last_el;
-				if (it->match(p, tmp_repl)) {
-					ops.erase(it);
-					goto found;
-				}
-				while(true) {
-					auto next_el = last_el;
-					++next_el;
-					if(next_el == repl_lst.end())
-						break;
-					else
-						repl_lst.remove_last();
-				}
-				++it;
-			}
-			return false; // no match found
-found:		;
-		}
-
-		if (has_global_wildcard) {
-
-			// Assign all the remaining terms to the global wildcard (unless
-			// it has already been matched before, in which case the matches
-			// must be equal)
-			std::unique_ptr<epvector> vp(new epvector);
-			vp->reserve(ops.size());
-			for (const auto& term : ops)
-				vp->push_back(split_ex_to_pair(term));
-
-                        ex rest = thisexpairseq(std::move(vp), default_overall_coeff());
-                        for (const auto & elem : tmp_repl) {
-                                if (elem.op(0).is_equal(global_wildcard)) {
-                                    if (rest.is_equal(elem.op(1))) {
-                                        repl_lst = tmp_repl;
-                                        return true;
-                                        }
-                                return false;
-                                }
-                        }
-                        repl_lst = tmp_repl;
-			repl_lst.append(global_wildcard == rest);
-			return true;
-
-		} else {
-
-			// No global wildcard, then the match fails if there are any
-			// unmatched terms left
-			if (ops.empty()) {
-				repl_lst = tmp_repl;
-				return true;
-			}
-			return false;
-		}
-	}
-	return inherited::match(pattern, repl_lst);
-}
-
-ex expairseq::subs(const exmap & m, unsigned options) const
-{
-	std::unique_ptr<epvector> vp = subschildren(m, options);
-	if (vp.get() != nullptr)
-		return ex_to<basic>(thisexpairseq(std::move(vp), overall_coeff, (options & subs_options::no_index_renaming) == 0));
-	else if (((options & subs_options::algebraic) != 0u) && is_exactly_a<mul>(*this))
-		return static_cast<const mul *>(this)->algebraic_subs_mul(m, options);
-	else
-		return subs_one_level(m, options);
-}
-
 // returns total degree of this sequence
 numeric expairseq::calc_total_degree() const
 {
@@ -654,12 +539,31 @@ long expairseq::calchash() const
 ex expairseq::expand(unsigned options) const
 {
 	std::unique_ptr<epvector> vp = expandchildren(options);
-	if (vp.get() != nullptr)
+	if (vp != nullptr)
 		return thisexpairseq(std::move(vp), overall_coeff);
-	else {
-		// The terms have not changed, so it is safe to declare this expanded
-		return (options == 0) ? setflag(status_flags::expanded) : *this;
-	}
+
+        // The terms have not changed, so it is safe to declare this expanded
+        return (options == 0) ? setflag(status_flags::expanded) : *this;
+}
+
+ex expairseq::subs(const exmap & m, unsigned options) const
+{
+	std::unique_ptr<epvector> vp = subschildren(m, options);
+	if (vp != nullptr) {
+                const ex& ocs = overall_coeff.subs(m, options);
+                if (is_exactly_a<numeric>(ocs))
+                        return ex_to<basic>(thisexpairseq(std::move(vp),
+                                            ex_to<numeric>(ocs),
+                                            (options & subs_options::no_index_renaming) == 0));
+                else
+                        return ex_to<basic>(add(ocs, thisexpairseq(std::move(vp),
+                                            *_num0_p,
+                                            (options & subs_options::no_index_renaming) == 0)));
+        }
+	if (((options & subs_options::algebraic) != 0u) && is_exactly_a<mul>(*this))
+		return static_cast<const mul *>(this)->algebraic_subs_mul(m, options);
+	else
+		return subs_one_level(m, options);
 }
 
 //////////
@@ -818,19 +722,19 @@ void expairseq::construct_from_2_ex(const ex &lh, const ex &rh)
 			}
 #endif // EXPAIRSEQ_USE_HASHTAB
 			return;
-		} else {
+		} 
 #if EXPAIRSEQ_USE_HASHTAB
-			unsigned totalsize = ex_to<expairseq>(lh).seq.size()+1;
-			if (calc_hashtabsize(totalsize)!=0) {
-				construct_from_2_ex_via_exvector(lh, rh);
-			} else {
+                unsigned totalsize = ex_to<expairseq>(lh).seq.size()+1;
+                if (calc_hashtabsize(totalsize)!=0) {
+                        construct_from_2_ex_via_exvector(lh, rh);
+                } else {
 #endif // EXPAIRSEQ_USE_HASHTAB
-				construct_from_expairseq_ex(ex_to<expairseq>(lh), rh);
+                        construct_from_expairseq_ex(ex_to<expairseq>(lh), rh);
 #if EXPAIRSEQ_USE_HASHTAB
 			}
 #endif // EXPAIRSEQ_USE_HASHTAB
 			return;
-		}
+		
 	} else if (ex_to<basic>(rh).tinfo()==this->tinfo()) {
 #if EXPAIRSEQ_USE_HASHTAB
 		unsigned totalsize=ex_to<expairseq>(rh).seq.size()+1;
@@ -923,7 +827,7 @@ void expairseq::construct_from_2_expairseq(const expairseq &s1,
 				const numeric &newcoeff = ex_to<numeric>(first1->coeff).
 					add(ex_to<numeric>(first2->coeff));
 				if (!newcoeff.is_zero()) {
-					seq.push_back(expair(first1->rest,newcoeff));
+					seq.emplace_back(first1->rest,newcoeff);
 					if (expair_needs_further_processing(seq.end()-1)) {
 						needs_further_processing = true;
 					}
@@ -990,14 +894,15 @@ void expairseq::construct_from_expairseq_ex(const expairseq &s,
 			const numeric &newcoeff = ex_to<numeric>(first->coeff).
 			                           add(ex_to<numeric>(p.coeff));
 			if (!newcoeff.is_zero()) {
-				seq.push_back(expair(first->rest,newcoeff));
+				seq.emplace_back(first->rest,newcoeff);
 				if (expair_needs_further_processing(seq.end()-1))
 					needs_further_processing = true;
 			}
 			++first;
 			p_pushed = true;
 			break;
-		} else if (cmpval<0) {
+		}
+                if (cmpval<0) {
 			seq.push_back(*first);
 			++first;
 		} else {
@@ -1132,8 +1037,8 @@ void expairseq::make_flat(const epvector &v, bool do_index_renaming)
 			combine_overall_coeff(subseqref.overall_coeff,
 			                      ex_to<numeric>(elem.coeff));
                         for (const auto & elem2 : subseqref.seq)
-				seq.push_back(expair(elem2.rest,
-                                        ex_to<numeric>(elem2.coeff).mul_dyn(ex_to<numeric>(elem.coeff))));
+				seq.emplace_back(elem2.rest,
+                                        ex_to<numeric>(elem2.coeff).mul_dyn(ex_to<numeric>(elem.coeff)));
 		} else {
 			if (elem.is_canonical_numeric())
 				combine_overall_coeff(ex_to<numeric>(mf.handle_factor(elem.rest, _ex1)));
@@ -1145,7 +1050,7 @@ void expairseq::make_flat(const epvector &v, bool do_index_renaming)
 				else if (are_ex_trivially_equal(newrest, rest))
 					seq.push_back(elem);
 				else
-					seq.push_back(expair(newrest, elem.coeff));
+					seq.emplace_back(newrest, elem.coeff);
 			}
 		}
 	}
@@ -1184,8 +1089,7 @@ void expairseq::combine_same_terms_sorted_seq()
 				needs_further_processing = true;
 			must_copy = true;
 		} else {
-			if (not ex_to<numeric>(itin1->coeff).is_zero()
-                                or ex_to<numeric>(itin1->coeff).is_parent_pos_char()) {
+			if (not ex_to<numeric>(itin1->coeff).is_zero()) {
 				if (must_copy)
 					*itout = *itin1;
 				++itout;
@@ -1194,8 +1098,7 @@ void expairseq::combine_same_terms_sorted_seq()
 		}
 		++itin2;
 	}
-	if (not ex_to<numeric>(itin1->coeff).is_zero()
-                or ex_to<numeric>(itin1->coeff).is_parent_pos_char()) {
+	if (not ex_to<numeric>(itin1->coeff).is_zero()) {
 		if (must_copy)
 			*itout = *itin1;
 		++itout;
@@ -1695,11 +1598,13 @@ std::unique_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned opti
 
 	} else {
 
-		// Substitute only in the "rest" part of the pairs
 		auto cit = seq.begin(), last = seq.end();
 		while (cit != last) {
-			const ex &subsed_ex = cit->rest.subs(m, options);
-			if (!are_ex_trivially_equal(cit->rest, subsed_ex)) {
+                        const ex &subsed_exr = cit->rest.subs(m, options);
+                        const ex &subsed_exc = ex_to<numeric>(cit->coeff).subs(m, options);
+                        if (not are_ex_trivially_equal(cit->rest, subsed_exr)
+                            or not are_ex_trivially_equal(cit->coeff,
+                                    subsed_exc)) {
 
                                 // Something changed, copy seq, subs and return it
 				std::unique_ptr<epvector> s(new epvector);
@@ -1709,14 +1614,21 @@ std::unique_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned opti
 				s->insert(s->begin(), seq.begin(), cit);
 			
 				// Copy first changed element
-				s->push_back(combine_ex_with_coeff_to_pair(subsed_ex,
-                                                        ex_to<numeric>(cit->coeff)));
+                                if (is_exactly_a<numeric>(subsed_exc))
+        				s->push_back(combine_ex_with_coeff_to_pair(subsed_exr, ex_to<numeric>(subsed_exc)));
+                                else
+        				s->push_back(split_ex_to_pair(mul(subsed_exr, subsed_exc)));
 				++cit;
 
 				// Copy rest
 				while (cit != last) {
-					s->push_back(combine_ex_with_coeff_to_pair(cit->rest.subs(m, options), ex_to<numeric>(cit->coeff)));
-					++cit;
+                                        const ex &sr = cit->rest.subs(m, options);
+                                        const ex &sc = ex_to<numeric>(cit->coeff).subs(m, options);
+					if (is_exactly_a<numeric>(sc))
+                                                s->push_back(combine_ex_with_coeff_to_pair(sr, ex_to<numeric>(sc)));
+                                        else
+                                                s->push_back(split_ex_to_pair(mul(sr, sc)));
+                                        ++cit;
 				}
 				return s;
 			}
@@ -1734,8 +1646,19 @@ const epvector & expairseq::get_sorted_seq() const
 {
 	if (seq_sorted.empty())
 		return seq;
-	else
-		return seq_sorted;
+
+        return seq_sorted;
+}
+
+bool expairseq::match(const ex & pattern, exmap& map) const
+{
+        CMatcher::level=0;
+        CMatcher cm(*this, pattern, map);
+        opt_exmap m = cm.get();
+        if (not m)
+                return false;
+        map = m.value();
+        return true;
 }
 
 //////////
