@@ -741,7 +741,7 @@ static ex asinh_conjugate(const ex & x)
 {
 	// conjugate(asinh(x))==asinh(conjugate(x)) unless on the branch cuts which
 	// run along the imaginary axis outside the interval [-I, +I].
-	if (x.info(info_flags::real))
+	if (x.is_real())
 		return asinh(x);
 	if (is_exactly_a<numeric>(x)) {
 		const numeric x_re = ex_to<numeric>(x.real_part());
@@ -848,10 +848,6 @@ static ex atanh_eval(const ex & x)
 		if (num.is_minus_one())
 			return NegInfinity;
 
-                // atanh(real) outside (-1,1)
-                if (num.is_real() and abs(num) > *_num1_p)
-                        return NaN;
-
 		// atanh() is odd
 		if (num.is_negative())
 			return -atanh(-x);
@@ -859,8 +855,11 @@ static ex atanh_eval(const ex & x)
 		// atanh(float) -> float
 		if (not num.is_exact())
 			return atanh(num);
-                else
-                        return atanh(x).hold();
+                
+                if (num.is_integer() or num.is_rational())
+                        return _ex1_2 * log(ex((*_num1_p + num)/(*_num1_p - num)));
+
+                return atanh(x).hold();
 	}
 	
 	// atanh(oo) -> -i*pi/2
@@ -900,9 +899,9 @@ static ex atanh_series(const ex &arg,
 	//     (log(1+x)-log(1-x))/2
 	// instead.
 	const ex arg_pt = arg.subs(rel, subs_options::no_pattern);
-	if (!(arg_pt).info(info_flags::real))
+	if (!(arg_pt).is_real())
 		throw do_taylor();     // Im(x) != 0
-	if ((arg_pt).info(info_flags::real) && abs(arg_pt)<_ex1)
+	if ((arg_pt).is_real() && abs(arg_pt)<_ex1)
 		throw do_taylor();     // Im(x) == 0, but abs(x)<1
 	// care for the poles, using the defining formula for atanh()...
 	if (arg_pt.is_equal(_ex1) || arg_pt.is_equal(_ex_1))
@@ -922,8 +921,8 @@ static ex atanh_series(const ex &arg,
 		else
 			Order0correction += log((arg_pt+_ex1)/(arg_pt+_ex_1))*_ex_1_2;
  		epvector seq;
-		seq.push_back(expair(Order0correction, _ex0));
- 		seq.push_back(expair(Order(_ex1), order));
+		seq.emplace_back(Order0correction, _ex0);
+ 		seq.emplace_back(Order(_ex1), order);
  		return series(replarg - pseries(rel, seq), rel, order);
 	}
 	throw do_taylor();
@@ -953,18 +952,21 @@ REGISTER_FUNCTION(atanh, eval_func(atanh_eval).
 static ex acoth_eval(const ex & x)
 {
         if (is_exactly_a<numeric>(x)) {
+                const numeric& num = ex_to<numeric>(x);
                 // acoth(1) -> oo
-                if (x.is_one())
+                if (num.is_one())
                         return Infinity;
                 // acoth(-1) -> -oo
-                if (x.is_minus_one())
+                if (num.is_minus_one())
                         return NegInfinity;
                 //acoth(float) -> float 
-                if (x.info(info_flags::inexact))
-                        return atanh(ex_to<numeric>(x).inverse());
+                if (not num.is_exact())
+                        return atanh(num.inverse());
                 // acoth() is odd
-                if (x.info(info_flags::negative))
-                        return -acoth(-x);
+                if (num.is_negative())
+                        return -acoth(num.negative());
+                if (num.is_integer() or num.is_rational())
+                        return _ex1_2 * log(ex((num + *_num1_p)/(num - *_num1_p)));
         }
        
 	if (is_exactly_a<function>(x)) {
@@ -1053,7 +1055,7 @@ static ex acsch_conjugate(const ex & x)
 {
         // conjugate(acsch(x))==acsch(conjugate(x)) unless on the branch cuts which
         // run along the imaginary axis inside the interval [-I, +I].
-        if (x.info(info_flags::real))
+        if (x.is_real())
 		return acsch(x);
 	if (is_exactly_a<numeric>(x)) {
 		const numeric x_re = ex_to<numeric>(x.real_part());
