@@ -69,7 +69,7 @@ static ex exp_eval(const ex & x)
 	// exp(-oo) -> 0
 	// exp(UnsignedInfinity) -> error
 	if (is_exactly_a<infinity>(x)) {
-	        infinity xinf = ex_to<infinity>(x);
+	        const infinity& xinf = ex_to<infinity>(x);
 		if (xinf.is_plus_infinity())
 			return Infinity;
 		if (xinf.is_minus_infinity())
@@ -104,11 +104,11 @@ static ex exp_eval(const ex & x)
 		ex rem = _ex0;
 		if (is_exactly_a<add>(coef_pi)) {
 			for (size_t i=0; i < coef_pi.nops(); i++) {
-				if ((coef_pi.op(i) / (_ex2 * I)).info(info_flags::integer))
+				if ((coef_pi.op(i) / (_ex2 * I)).is_integer())
 					rem += Pi * coef_pi.op(i);
 			}
 		}
-		else if ((coef_pi / (_ex2 * I)).info(info_flags::integer))
+		else if ((coef_pi / (_ex2 * I)).is_integer())
 			rem = Pi * coef_pi;
 		x_red = (x - rem).expand();
 
@@ -124,10 +124,82 @@ static ex exp_eval(const ex & x)
 	else
 		x_red = x;
 
-	// exp(log(x)) -> x
-	if (is_ex_the_function(x_red, log))
-		return x_red.op(0);
-	
+        if (is_exactly_a<function>(x_red)) {
+                const ex& arg = x_red.op(0);
+                // exp(log(x)) -> x
+                if (is_ex_the_function(x_red, log))
+                        return arg;
+                // exp(asinh(num)) etc.
+                if (is_exactly_a<numeric>(arg)) {
+                        if (is_ex_the_function(x_red, asinh))
+                                return arg + sqrt(power(arg, _ex2) + _ex1);
+                        if (is_ex_the_function(x_red, acosh))
+                                return arg + sqrt(power(arg, _ex2) - _ex1);
+                        if (is_ex_the_function(x_red, atanh))
+                                return sqrt((arg+_ex1) / (arg-_ex1));
+                        if (is_ex_the_function(x_red, acoth))
+                                return sqrt((arg-_ex1) / (arg+_ex1));
+                        if (is_ex_the_function(x_red, asech))
+                                return (_ex1/arg +
+                                        sqrt(_ex1-power(arg, _ex2))/arg);
+                        if (is_ex_the_function(x_red, acsch))
+                                return (_ex1/arg +
+                                        sqrt(_ex1+power(arg, _ex2))/arg);
+                }
+        }
+
+        static std::unordered_set<unsigned int> funcs = { log_SERIAL::serial,
+                asinh_SERIAL::serial, acosh_SERIAL::serial,
+                atanh_SERIAL::serial, acoth_SERIAL::serial,
+                asech_SERIAL::serial, acsch_SERIAL::serial };
+
+        if (is_exactly_a<mul>(x_red)) {
+                // Loop through factors finding and marking the function.
+                // Don't proceed if more than one function found.
+                bool function_seen = false, applicable = false;
+                size_t func_idx;
+                for (size_t i=0; i<x_red.nops(); ++i) {
+                        const ex& fac = x_red.op(i);
+                        if (is_exactly_a<function>(fac)
+                            and funcs.find(ex_to<function>(fac).get_serial()) != funcs.end()) {
+                                if (function_seen)
+                                        { applicable = false; break; }
+                                function_seen = applicable = true;
+                                func_idx = i;
+                        }
+                }
+                if (applicable) {
+                        ex c = _ex1;
+                        for (size_t i=0; i<x_red.nops(); ++i)
+                                if (i != func_idx)
+                                        c *= x_red.op(i);
+                        const ex& fac = x_red.op(func_idx);
+                        const ex& arg = fac.op(0);
+                        // exp(c*log(ex)) -> ex^c
+                        if (is_ex_the_function(fac, log))
+                                return power(arg, c);
+                        // exp(c*asinh(ex)) etc.
+                        if (is_ex_the_function(fac, asinh))
+                                return power(arg + sqrt(power(arg, _ex2) + 1),
+                                                c);
+                        if (is_ex_the_function(fac, acosh))
+                                return power(arg + sqrt(power(arg, _ex2) - 1),
+                                                c);
+                        if (is_ex_the_function(fac, atanh))
+                                return power((arg+_ex1) / (arg-_ex1), c/2);
+                        if (is_ex_the_function(fac, acoth))
+                                return power((arg-_ex1) / (arg+_ex1), c/2);
+                        if (is_ex_the_function(fac, asech))
+                                return power(_ex1/arg +
+                                        sqrt(_ex1-power(arg, _ex2))/arg,
+                                        c);
+                        if (is_ex_the_function(fac, acsch))
+                                return power(_ex1/arg +
+                                        sqrt(_ex1+power(arg, _ex2))/arg,
+                                        c);
+                }
+        }
+
 	// exp(float) -> float
 	if (is_exactly_a<numeric>(x_red)
             and x_red.info(info_flags::inexact))
@@ -160,8 +232,8 @@ static ex exp_power(const ex & arg, const ex & p)
 	// the simplification code in mul::eval accordingly
 	if (is_exactly_a<numeric>(p) && ex_to<numeric>(p).is_integer())
 		return exp(p*arg);
-	else
-		return power(exp(arg), p).hold();
+
+	return power(exp(arg), p).hold();
 }
 
 static void exp_print(const ex & arg, const print_context & c,
@@ -175,7 +247,7 @@ static void exp_print(const ex & arg, const print_context & c,
 		if (latex) {
 			tcontext_p.reset(new print_latex(tstream, c.options));
 		} else {
-			tcontext_p.reset(new print_dflt(tstream, c.options));
+			tcontext_p.reset(new print_context(tstream, c.options));
 		}
 		arg.print(*tcontext_p);
 		std::string argstr = tstream.str();
@@ -202,7 +274,7 @@ static void exp_print(const ex & arg, const print_context & c,
 	}
 }
 
-static void exp_print_dflt(const ex & arg, const print_context & c)
+static void exp_print_norm(const ex & arg, const print_context & c)
 {
 	exp_print(arg, c, false);
 }
@@ -224,7 +296,7 @@ REGISTER_FUNCTION(exp, eval_func(exp_eval).
                        imag_part_func(exp_imag_part).
                        power_func(exp_power).
                        conjugate_func(exp_conjugate).
-                       print_func<print_dflt>(exp_print_dflt).
+                       print_func<print_context>(exp_print_norm).
                        print_func<print_latex>(exp_print_latex));
 
 //////////
@@ -235,7 +307,7 @@ static ex log_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
 		// log(float) -> float
-                numeric n = ex_to<numeric>(x);
+                const numeric& n = ex_to<numeric>(x);
 		if (x.info(info_flags::crational)) {
                         if (n.is_zero())         // log(0) -> infinity
                                 //throw(pole_error("log_eval(): log(0)",0));
@@ -262,7 +334,7 @@ static ex log_eval(const ex & x)
 	// log(exp(t)) -> t (if -Pi < t.imag() <= Pi):
 	if (is_ex_the_function(x, exp)) {
 		const ex &t = x.op(0);
-		if (t.info(info_flags::real))
+		if (t.is_real())
 			return t;
 	}
 	
@@ -330,10 +402,10 @@ static ex log_series(const ex &arg,
 		const ex coeff = argser.coeff(s, n);
 		// expand the log, but only if coeff is real and > 0, since otherwise
 		// it would make the branch cut run into the wrong direction
-		if (coeff.info(info_flags::positive))
-			seq.push_back(expair(n*log(s-point)+log(coeff), _ex0));
+		if (coeff.is_positive())
+			seq.emplace_back(n*log(s-point)+log(coeff), _ex0);
 		else
-			seq.push_back(expair(log(coeff*pow(s-point, n)), _ex0));
+			seq.emplace_back(log(coeff*pow(s-point, n)), _ex0);
 
 		if (!argser.is_terminating() || argser.nops()!=1) {
 			// in this case n more (or less) terms are needed
@@ -342,13 +414,13 @@ static ex log_series(const ex &arg,
 				epvector epv;
 				ex acc = (new pseries(rel, epv))->setflag(status_flags::dynallocated);
 				epv.reserve(2);
-				epv.push_back(expair(-1, _ex0));
-				epv.push_back(expair(Order(_ex1), order));
+				epv.emplace_back(-1, _ex0);
+				epv.emplace_back(Order(_ex1), order);
 				ex rest = pseries(rel, epv).add_series(argser);
 				for (int i = order-1; i>0; --i) {
 					epvector cterm;
 					cterm.reserve(1);
-					cterm.push_back(expair((i%2) != 0 ? _ex1/i : _ex_1/i, _ex0));
+					cterm.emplace_back((i%2) != 0 ? _ex1/i : _ex_1/i, _ex0);
 					acc = pseries(rel, cterm).add_series(ex_to<pseries>(acc));
 					acc = (ex_to<pseries>(rest)).mul_series(ex_to<pseries>(acc));
 				}
@@ -356,8 +428,8 @@ static ex log_series(const ex &arg,
 			}
 			const ex newarg = ex_to<pseries>((arg/coeff).series(rel, order+n, options)).shift_exponents(-n).convert_to_poly(true);
 			return pseries(rel, seq).add_series(ex_to<pseries>(log(newarg).series(rel, order, options)));
-		} else  // it was a monomial
-			return pseries(rel, seq);
+		}  // it was a monomial
+		return pseries(rel, seq);
 	}
 	if (((options & series_options::suppress_branchcut) == 0u) &&
 	     arg_pt.info(info_flags::negative)) {
@@ -369,8 +441,8 @@ static ex log_series(const ex &arg,
 		const symbol foo;
 		const ex replarg = series(log(arg), s==foo, order).subs(foo==point, subs_options::no_pattern);
 		epvector seq;
-		seq.push_back(expair(-I*csgn(arg*I)*Pi, _ex0));
-		seq.push_back(expair(Order(_ex1), order));
+		seq.emplace_back(-I*csgn(arg*I)*Pi, _ex0);
+		seq.emplace_back(Order(_ex1), order);
 		return series(replarg - I*Pi + pseries(rel, seq), rel, order);
 	}
 	throw do_taylor();  // caught by function::series()
@@ -378,14 +450,14 @@ static ex log_series(const ex &arg,
 
 static ex log_real_part(const ex & x)
 {
-	if (x.info(info_flags::positive))
+	if (x.is_positive())
 		return log(x).hold();
 	return log(abs(x));
 }
 
 static ex log_imag_part(const ex & x)
 {
-	if (x.info(info_flags::positive))
+	if (x.is_positive())
 		return _ex0;
 	return atan2(GiNaC::imag_part(x), GiNaC::real_part(x));
 }
@@ -394,7 +466,7 @@ static ex log_conjugate(const ex & x)
 {
 	// conjugate(log(x))==log(conjugate(x)) unless on the branch cut which
 	// runs along the negative real axis.
-	if (x.info(info_flags::positive)) {
+	if (x.is_positive()) {
 		return log(x);
 	}
 	if (is_exactly_a<numeric>(x) &&
@@ -422,8 +494,7 @@ static ex logb_evalf(const ex & x, const ex & base, PyObject* parent)
         if ((base - exp(_ex1).hold()).is_zero()) {
                 if (is_exactly_a<numeric>(x))
                         return log(ex_to<numeric>(x), parent);
-                else
-                        return log(x);
+                return log(x);
         }
 	if (is_exactly_a<numeric>(x) and is_exactly_a<numeric>(base))
 		return log(ex_to<numeric>(x), ex_to<numeric>(base));
@@ -435,9 +506,9 @@ static ex logb_eval(const ex & x, const ex & base)
 {
 	if (is_exactly_a<numeric>(x) and not x.info(info_flags::inexact)
 	    and is_exactly_a<numeric>(base) and not base.info(info_flags::inexact)) {
-                numeric a = ex_to<numeric>(x);
-                numeric b = ex_to<numeric>(base);
-                if (b.info(info_flags::real) and a.info(info_flags::real)) {
+                const numeric& a = ex_to<numeric>(x);
+                const numeric& b = ex_to<numeric>(base);
+                if (b.is_real() and a.is_real()) {
                         bool israt;
                         numeric ret = a.ratlog(b, israt);
                         if (israt)
@@ -451,7 +522,7 @@ static ex logb_eval(const ex & x, const ex & base)
 
 	// log(base^t, base) -> t
 	if (is_exactly_a<power>(x)) {
-                if (x.op(0).is_equal(base) and x.op(1).info(info_flags::real))
+                if (x.op(0).is_equal(base) and x.op(1).is_real())
 			return x.op(1);
 	}
 
@@ -477,17 +548,7 @@ static ex Li2_evalf(const ex & x, PyObject* parent)
 	if (not is_exactly_a<numeric>(x))
 	        return Li2(x).hold();
 
-        const numeric& num = ex_to<numeric>(x);
-        try {
-                return numeric::try_py_method(num, "dilog");
-        }
-        catch (std::logic_error) {}
-        try {
-                return numeric::try_py_method(num, "polylog", *_num2_p);
-        }
-        catch (std::logic_error) {}
-
-        return Li2(num, parent);
+        return Li2(ex_to<numeric>(x), parent);
 }
 
 static ex Li2_eval(const ex & x)
@@ -557,7 +618,7 @@ static ex Li2_series(const ex &x, const relational &rel, int order, unsigned opt
 			ser = ser.subs(s==x.series(rel, order), subs_options::no_pattern);
 			// maybe that was terminating, so add a proper order term
 			epvector nseq;
-			nseq.push_back(expair(Order(_ex1), order));
+			nseq.emplace_back(Order(_ex1), order);
 			ser += pseries(rel, nseq);
 			// reexpanding it will collapse the series again
 			return ser.series(rel, order);
@@ -582,7 +643,7 @@ static ex Li2_series(const ex &x, const relational &rel, int order, unsigned opt
 			ser = ser.subs(s==x.series(rel, order), subs_options::no_pattern);
 			// maybe that was terminating, so add a proper order term
 			epvector nseq;
-			nseq.push_back(expair(Order(_ex1), order));
+			nseq.emplace_back(Order(_ex1), order);
 			ser += pseries(rel, nseq);
 			// reexpanding it will collapse the series again
 			return ser.series(rel, order);
@@ -598,18 +659,18 @@ static ex Li2_series(const ex &x, const relational &rel, int order, unsigned opt
 			const symbol foo;
 			epvector seq;
 			// zeroth order term:
-			seq.push_back(expair(Li2(x_pt), _ex0));
+			seq.emplace_back(Li2(x_pt), _ex0);
 			// compute the intermediate terms:
 			ex replarg = series(Li2(x), s==foo, order);
 			for (unsigned i=1; i < replarg.nops()-1; ++i) {
 				ex term = replarg.op(i) / power(s-foo, i);
                                 term = term.series(foo==point,1,options).op(0);
                                 term.subs(foo==s, subs_options::no_pattern);
-				seq.push_back(expair(term, numeric(i)));
+				seq.emplace_back(term, numeric(i));
                         }
 			// append an order term:
-			seq.push_back(expair(Order(_ex1),
-                                                long(replarg.nops()-1)));
+			seq.emplace_back(Order(_ex1),
+                                                long(replarg.nops()-1));
 			return pseries(rel, seq);
 		}
 	}
@@ -653,11 +714,6 @@ static ex Li_evalf(const ex& m_, const ex& x_, PyObject* parent)
         const numeric& num_m = ex_to<numeric>(m_);
         const numeric& num_x = ex_to<numeric>(x_);
         
-        try {
-                return numeric::try_py_method(num_x, "polylog", num_m);
-        }
-        catch (std::logic_error) {}
-
         return Li2(num_m, num_x, parent);
 }
 
@@ -710,7 +766,7 @@ static ex Li_series(const ex& m, const ex& x, const relational& rel, int order,
 			ser = ser.subs(s==x.series(rel, order), subs_options::no_pattern);
 			// maybe that was terminating, so add a proper order term
 			epvector nseq;
-			nseq.push_back(expair(Order(_ex1), order));
+			nseq.emplace_back(Order(_ex1), order);
 			ser += pseries(rel, nseq);
 			// reexpanding it will collapse the series again
 			return ser.series(rel, order);
@@ -729,13 +785,12 @@ static ex Li_deriv(const ex& m_, const ex& x_, unsigned deriv_param)
 	if (deriv_param == 0) {
 		return _ex0;
 	}
-	ex m = m_;
-	ex x = x_;
+	const ex& m = m_;
+	const ex& x = x_;
 	if (m > 0) {
 		return Li(m-1, x) / x;
-	} else {
-		return 1/(1-x);
-	}
+	} 
+	return 1/(1-x);
 }
 
 
