@@ -101,11 +101,11 @@ static ex sin_eval(const ex & x)
                 ex rem = _ex0;
                 if (is_exactly_a<add>(coef_pi)) {
                         for (size_t i=0; i < coef_pi.nops(); i++) {
-                                if ((coef_pi.op(i) / _ex2).info(info_flags::integer))
+                                if ((coef_pi.op(i) / _ex2).is_integer())
                                         rem += Pi * coef_pi.op(i);
                         }
                 }
-                else if ((coef_pi / _ex2).info(info_flags::integer))
+                else if ((coef_pi / _ex2).is_integer())
                         rem = Pi * coef_pi;
                 x_red = (x - rem).expand();
 
@@ -113,7 +113,7 @@ static ex sin_eval(const ex & x)
                 const ex SixtyExOverPi = _ex60*x_red/Pi;
                 ex sign = _ex1;
                 if (is_exactly_a<numeric>(SixtyExOverPi)
-                        and SixtyExOverPi.info(info_flags::integer)) {
+                        and SixtyExOverPi.is_integer()) {
                         numeric z = mod(ex_to<numeric>(SixtyExOverPi),*_num120_p);
                         if (z>=*_num60_p) {
                                 // wrap to interval [0, Pi)
@@ -164,7 +164,7 @@ static ex sin_eval(const ex & x)
                 const ex TwentyforExOverPi = _ex24*x_red/Pi;
                 sign = _ex1;
                 if (is_exactly_a<numeric>(TwentyforExOverPi)
-                        and TwentyforExOverPi.info(info_flags::integer)) {
+                        and TwentyforExOverPi.is_integer()) {
                         numeric z = mod(ex_to<numeric>(TwentyforExOverPi),*_num48_p);
                         if (z>=*_num24_p) {
                                 // wrap to interval [0, Pi)
@@ -191,10 +191,10 @@ static ex sin_eval(const ex & x)
                 
                 // Reflection at Pi/2
                 const ex ExOverPi = x_red/Pi;
-                if (ExOverPi.info(info_flags::integer))
+                if (ExOverPi.is_integer())
                         return _ex0;
                 if (is_exactly_a<numeric>(ExOverPi)) {
-                        const numeric c = ex_to<numeric>(ExOverPi);
+                        const numeric& c = ex_to<numeric>(ExOverPi);
                         if (c.is_rational()) {
                                 numeric den = c.denom();
                                 numeric num = c.numer().mod(den * *_num2_p);
@@ -205,8 +205,7 @@ static ex sin_eval(const ex & x)
                                 }
                                 if (num*(*_num2_p) > den)
                                         return fac*sin_eval((den-num)*Pi/den);
-                                else
-                                        return fac*sin((num*Pi)/den).hold();
+                                return fac*sin((num*Pi)/den).hold();
                         }
                 }
         }
@@ -316,11 +315,11 @@ static ex cos_eval(const ex & x)
                 ex rem = _ex0;
                 if (is_exactly_a<add>(coef_pi)) {
                         for (size_t i=0; i < coef_pi.nops(); i++) {
-                                if ((coef_pi.op(i) / _ex2).info(info_flags::integer))
+                                if ((coef_pi.op(i) / _ex2).is_integer())
                                         rem += Pi * coef_pi.op(i);
                         }
                 }
-                else if ((coef_pi / _ex2).info(info_flags::integer))
+                else if ((coef_pi / _ex2).is_integer())
                         rem = Pi * coef_pi;
                 x_red = (x - rem).expand();
 
@@ -328,7 +327,7 @@ static ex cos_eval(const ex & x)
                 const ex SixtyExOverPi = _ex60*x_red/Pi;
                 ex sign = _ex1;
                 if (is_exactly_a<numeric>(SixtyExOverPi)
-                        and SixtyExOverPi.info(info_flags::integer)) {
+                        and SixtyExOverPi.is_integer()) {
                         numeric z = mod(ex_to<numeric>(SixtyExOverPi),*_num120_p);
                         if (z>=*_num60_p) {
                                 // wrap to interval [0, Pi)
@@ -374,7 +373,7 @@ static ex cos_eval(const ex & x)
                 const ex TwentyforExOverPi = _ex24*x_red/Pi;
                 sign = _ex1;
                 if (is_exactly_a<numeric>(TwentyforExOverPi)
-                                and TwentyforExOverPi.info(info_flags::integer)) {
+                                and TwentyforExOverPi.is_integer()) {
                         numeric z = mod(ex_to<numeric>(TwentyforExOverPi),*_num48_p);
                         if (z>=*_num24_p) {
                                 // wrap to interval [0, Pi)
@@ -403,9 +402,9 @@ static ex cos_eval(const ex & x)
                 // Reflection at Pi/2
                 const ex ExOverPi = x_red/Pi;
                 if (is_exactly_a<numeric>(ExOverPi)) {
-                        const numeric c = ex_to<numeric>(ExOverPi);
+                        const numeric& c = ex_to<numeric>(ExOverPi);
                         // cos(integer*pi) --> (-1)^integer
-                        if (c.info(info_flags::integer))
+                        if (c.is_integer())
                                 return pow(*_num_1_p, c);
                         if (c.is_rational()) {
                                 numeric den = c.denom();
@@ -414,7 +413,7 @@ static ex cos_eval(const ex & x)
                                         num = den * *_num2_p - num;
                                 if (num*(*_num2_p) > den)
                                         return mul(_ex_1, cos_eval((den-num)*Pi/den));
-                                else
+                                
                                         return cos((num*Pi)/den).hold();
                         }
                 }
@@ -526,11 +525,11 @@ static ex tan_eval(const ex & x)
                 ex rem = _ex0;
                 if (is_exactly_a<add>(coef_pi)) {
                         for (size_t i=0; i < coef_pi.nops(); i++) {
-                                if (coef_pi.op(i).info(info_flags::integer))
+                                if (coef_pi.op(i).is_integer())
                                         rem += Pi * coef_pi.op(i);
                         }
                 }
-                else if (coef_pi.info(info_flags::integer))
+                else if (coef_pi.is_integer())
                         rem = Pi * coef_pi;
                 x_red = (x - rem).expand();
 
@@ -539,7 +538,7 @@ static ex tan_eval(const ex & x)
                 const ex SixtyExOverPi = _ex60*x_red/Pi;
                 ex sign = _ex1;
                 if (is_exactly_a<numeric>(SixtyExOverPi)
-                        and SixtyExOverPi.info(info_flags::integer)) {
+                        and SixtyExOverPi.is_integer()) {
                         numeric z = mod(ex_to<numeric>(SixtyExOverPi),*_num60_p);
                         if (z>=*_num60_p) {
                                 // wrap to interval [0, Pi)
@@ -586,7 +585,7 @@ static ex tan_eval(const ex & x)
                 const ex FortyeightExOverPi = _ex48*x_red/Pi;
                 sign = _ex1;
                 if (is_exactly_a<numeric>(FortyeightExOverPi)
-                                and FortyeightExOverPi.info(info_flags::integer)) {
+                                and FortyeightExOverPi.is_integer()) {
                         numeric z = mod(ex_to<numeric>(FortyeightExOverPi),*_num48_p);
                         if (z>=*_num48_p) {
                                 // wrap to interval [0, Pi)
@@ -622,14 +621,13 @@ static ex tan_eval(const ex & x)
                 // Reflection at Pi/2
                 const ex ExOverPi = x_red/Pi;
                 if (is_exactly_a<numeric>(ExOverPi)) {
-                        const numeric c = ex_to<numeric>(ExOverPi);
+                        const numeric& c = ex_to<numeric>(ExOverPi);
                         if (c.is_rational()) {
                                 numeric den = c.denom();
                                 numeric num = c.numer().mod(den);
                                 if (num*(*_num2_p) > den)
                                         return mul(_ex_1, tan((den-num)*Pi/den).hold());
-                                else
-                                        return tan((num*Pi)/den).hold();
+                                return tan((num*Pi)/den).hold();
                         }
                 }
         }
@@ -794,7 +792,7 @@ static ex cot_eval(const ex & x)
                 if (not res.is_zero()) {     
 			return tan_eval(Pi/2-x);
                 }
-                else
+                
                         return UnsignedInfinity;
         }
         // Reflection at Pi/2
@@ -802,7 +800,7 @@ static ex cot_eval(const ex & x)
         if(is_exactly_a<numeric>(ExOverPi)) {
 		ex coef_pi = x.coeff(Pi,_ex1).expand();
 		if (is_exactly_a<numeric>(coef_pi)) {
-			const numeric c = ex_to<numeric>(coef_pi);
+			const numeric& c = ex_to<numeric>(coef_pi);
 		        if (c.is_rational()) {
 	                        const numeric num = c.numer();
 		                const numeric den = c.denom();
@@ -932,13 +930,12 @@ static ex sec_eval(const ex & x)
         if (not is_ex_the_function(res, cos) && not is_ex_the_function(_ex_1*res, cos)) {
                 if (res.is_zero())
                         return UnsignedInfinity;
-                else
-                        return power(res, _ex_1);
+                return power(res, _ex_1);
         }
         // cos has reflected also the argument so take it
         if (is_ex_the_function(res, cos))
                 return sec(res.op(0)).hold();
-        else
+        
                 return -sec((-res).op(0)).hold();
 }
 
@@ -1046,14 +1043,13 @@ static ex csc_eval(const ex & x)
         if (not is_ex_the_function(res, sin) && not is_ex_the_function(_ex_1*res, sin)) {
                 if (res.is_zero())
                         return UnsignedInfinity;
-                else
+                
                         return power(res, _ex_1);
         }
         // sin has reflected also the argument so take it
         if (is_ex_the_function(res, sin))
                 return csc(res.op(0)).hold();
-        else
-                return -csc((-res).op(0)).hold();
+        return -csc((-res).op(0)).hold();
 }
 
 static ex csc_deriv(const ex & x, unsigned deriv_param)
@@ -1107,10 +1103,22 @@ REGISTER_FUNCTION(csc, eval_func(csc_eval).
 // inverse sine (arc sine)
 //////////
 
+// Needed because there is no RR member
+static ex asin_evalf(const ex & x, PyObject* parent)
+{
+	if (is_exactly_a<numeric>(x))
+		return asin(ex_to<numeric>(x), parent);
+
+	return asin(x).hold();
+}
+
 static ex asin_eval(const ex & x)
 {
-	if (is_exactly_a<numeric>(x)) {
+        // asin() is odd
+        if (x.info(info_flags::negative))
+                return -asin(-x);
 
+	if (is_exactly_a<numeric>(x)) {
 		// asin(0) -> 0
 		if (x.is_zero())
 			return x;
@@ -1120,30 +1128,16 @@ static ex asin_eval(const ex & x)
 			return numeric(1,6)*Pi;
 
 		// asin(1) -> Pi/2
-		if (x.is_equal(_ex1))
+		if (x.is_one())
 			return _ex1_2*Pi;
 
-		// asin(-1/2) -> -Pi/6
-		if (x.is_equal(_ex_1_2))
-			return numeric(-1,6)*Pi;
-
-		// asin(-1) -> -Pi/2
-		if (x.is_equal(_ex_1))
-			return _ex_1_2*Pi;
-
 		if (x.info(info_flags::inexact)) {
                         const numeric& num = ex_to<numeric>(x);
-                        if (num.is_real()
-                            and (num < *_num_1_p or num > *_num1_p))
-                                return NaN;
-                        else
-        		        // asin(float) -> float
-	        		return asin(ex_to<numeric>(x));
+
+                        // asin(float) -> float
+                        return asin(num);
                 }
 
-		// asin() is odd
-		if (x.info(info_flags::negative))
-			return -asin(-x);
 	}
 	
 	// asin(oo) -> error
@@ -1154,6 +1148,12 @@ static ex asin_eval(const ex & x)
 		throw (std::runtime_error("arcsin_eval(): arcsin(infinity) encountered"));
 	}
 
+        if (x.is_equal(mul(pow(_ex2, _ex1_2), _ex1_2)))
+                return mul(Pi, _ex1_4);
+
+        if (x.is_equal(mul(pow(_ex3, _ex1_2), _ex1_2)))
+                return mul(Pi, _ex1_3);
+
 	return asin(x).hold();
 }
 
@@ -1177,6 +1177,7 @@ static ex asin_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(asin, eval_func(asin_eval).
+                        evalf_func(asin_evalf).
                         derivative_func(asin_deriv).
                         conjugate_func(asin_conjugate).
 			set_name("arcsin", "\\arcsin"));
@@ -1190,7 +1191,7 @@ static ex acos_eval(const ex & x)
 	if (is_exactly_a<numeric>(x)) {
 
 		// acos(1) -> 0
-		if (x.is_equal(_ex1))
+		if (x.is_one())
 			return _ex0;
 
 		// acos(1/2) -> Pi/3
@@ -1206,17 +1207,14 @@ static ex acos_eval(const ex & x)
 			return numeric(2,3)*Pi;
 
 		// acos(-1) -> Pi
-		if (x.is_equal(_ex_1))
+		if (x.is_minus_one())
 			return Pi;
 
 		if (x.info(info_flags::inexact)) {
                         const numeric& num = ex_to<numeric>(x);
-                        if (num.is_real()
-                            and (num < *_num_1_p or num > *_num1_p))
-                                return NaN;
-                        else
-        		        // acos(float) -> float
-	        		return acos(ex_to<numeric>(x));
+
+                        // acos(float) -> float
+                        return acos(num);
                 }
 
 		// acos(-x) -> Pi-acos(x)
@@ -1265,37 +1263,32 @@ REGISTER_FUNCTION(acos, eval_func(acos_eval).
 
 static ex atan_eval(const ex & x)
 {
-	if (is_exactly_a<numeric>(x)) {
+        // atan() is odd
+        if (x.info(info_flags::negative))
+                return -atan(-x);
 
+        if (is_exactly_a<numeric>(x)) {
 		// atan(0) -> 0
 		if (x.is_zero())
 			return _ex0;
 
 		// atan(1) -> Pi/4
-		if (x.is_equal(_ex1))
+		if (x.is_one())
 			return _ex1_4*Pi;
 
-		// atan(-1) -> -Pi/4
-		if (x.is_equal(_ex_1))
-			return _ex_1_4*Pi;
-
-		if (x.is_equal(I) || x.is_equal(-I))
+		if (x.is_equal(I))
 			throw (pole_error("atan_eval(): logarithmic pole",0));
 
 		// atan(float) -> float
 		if (x.info(info_flags::inexact))
 			return atan(ex_to<numeric>(x));
-
-		// atan() is odd
-		if (x.info(info_flags::negative))
-			return -atan(-x);
 	}
 	
 	// arctan(oo) -> Pi/2
 	// arctan(-oo) -> -Pi/2
 	// arctan(UnsignedInfinity) -> error
 	if (is_exactly_a<infinity>(x)) {
-	        infinity xinf = ex_to<infinity>(x);
+	        const infinity& xinf = ex_to<infinity>(x);
 		if (xinf.is_plus_infinity())
 		        return _ex1_2*Pi;
 		if (xinf.is_minus_infinity())
@@ -1303,7 +1296,13 @@ static ex atan_eval(const ex & x)
 		// x is UnsignedInfinity
 		throw (std::runtime_error("arctan_eval(): arctan(unsigned_infinity) encountered"));
 	}
-		
+
+        if (x.is_equal(pow(_ex3, _ex1_2)))
+                return mul(Pi, _ex1_3);
+
+        if (x.is_equal(mul(pow(_ex3, _ex1_2), _ex1_3)))
+                return mul(Pi, numeric(1,6));
+
 	return atan(x).hold();
 }
 
@@ -1330,9 +1329,9 @@ static ex atan_series(const ex &arg,
 	//     (log(1+I*x)-log(1-I*x))/(2*I)
 	// instead.
 	const ex arg_pt = arg.subs(rel, subs_options::no_pattern);
-	if (!(I*arg_pt).info(info_flags::real))
+	if (not (I*arg_pt).is_real())
 		throw do_taylor();     // Re(x) != 0
-	if ((I*arg_pt).info(info_flags::real) && abs(I*arg_pt)<_ex1)
+	if ((I*arg_pt).is_real() && abs(I*arg_pt)<_ex1)
 		throw do_taylor();     // Re(x) == 0, but abs(x)<1
 	// care for the poles, using the defining formula for atan()...
 	if (arg_pt.is_equal(I) || arg_pt.is_equal(-I))
@@ -1351,8 +1350,8 @@ static ex atan_series(const ex &arg,
 		else
 			Order0correction += log((I*arg_pt+_ex1)/(I*arg_pt+_ex_1))*I*_ex1_2;
 		epvector seq;
-		seq.push_back(expair(Order0correction, _ex0));
-		seq.push_back(expair(Order(_ex1), order));
+		seq.emplace_back(Order0correction, _ex0);
+		seq.emplace_back(Order(_ex1), order);
 		return series(replarg - pseries(rel, seq), rel, order);
 	}
 	throw do_taylor();
@@ -1362,7 +1361,7 @@ static ex atan_conjugate(const ex & x)
 {
 	// conjugate(atan(x))==atan(conjugate(x)) unless on the branch cuts which
 	// run along the imaginary axis outside the interval [-I, +I].
-	if (x.info(info_flags::real))
+	if (x.is_real())
 		return atan(x);
 	if (is_exactly_a<numeric>(x)) {
 		const numeric x_re = ex_to<numeric>(x.real_part());
@@ -1386,7 +1385,7 @@ REGISTER_FUNCTION(atan, eval_func(atan_eval).
 
 static ex atan2_evalf(const ex &y, const ex &x, PyObject* parent)
 {
-	if (is_exactly_a<numeric>(y) && is_exactly_a<numeric>(x))
+	if (is_exactly_a<numeric>(y) and is_exactly_a<numeric>(x))
 		return atan(ex_to<numeric>(y), ex_to<numeric>(x));
 	
 	return atan2(y, x).hold();
@@ -1401,7 +1400,7 @@ static ex atan2_eval(const ex & y, const ex & x)
 			return NaN;
 
 		// atan2(0, x), x real and positive -> 0
-		if (x.info(info_flags::positive))
+		if (x.is_positive())
 			return _ex0;
 
 		// atan2(0, x), x real and negative -> Pi
@@ -1412,7 +1411,7 @@ static ex atan2_eval(const ex & y, const ex & x)
 	if (x.is_zero()) {
 
 		// atan2(y, 0), y real and positive -> Pi/2
-		if (y.info(info_flags::positive))
+		if (y.is_positive())
 			return _ex1_2*Pi;
 
 		// atan2(y, 0), y real and negative -> -Pi/2
@@ -1423,7 +1422,7 @@ static ex atan2_eval(const ex & y, const ex & x)
 	if (y.is_equal(x)) {
 
 		// atan2(y, y), y real and positive -> Pi/4
-		if (y.info(info_flags::positive))
+		if (y.is_positive())
 			return _ex1_4*Pi;
 
 		// atan2(y, y), y real and negative -> -3/4*Pi
@@ -1434,7 +1433,7 @@ static ex atan2_eval(const ex & y, const ex & x)
 	if (y.is_equal(-x)) {
 
 		// atan2(y, -y), y real and positive -> 3*Pi/4
-		if (y.info(info_flags::positive))
+		if (y.is_positive())
 			return numeric(3, 4)*Pi;
 
 		// atan2(y, -y), y real and negative -> -Pi/4
@@ -1443,8 +1442,9 @@ static ex atan2_eval(const ex & y, const ex & x)
 	}
 
 	// atan2(float, float) -> float
-	if (is_exactly_a<numeric>(y) && y.info(info_flags::inexact) &&
-	    is_exactly_a<numeric>(x) && x.info(info_flags::inexact))
+	if (is_exactly_a<numeric>(x)
+            and is_exactly_a<numeric>(y)
+            and (x.info(info_flags::inexact) or y.info(info_flags::inexact)))
 		return atan(ex_to<numeric>(y), ex_to<numeric>(x));
 
 	// handle infinities
@@ -1465,12 +1465,12 @@ static ex atan2_eval(const ex & y, const ex & x)
 	}
 
 	// atan2(real, real) -> atan(y/x) +/- Pi
-	if (y.info(info_flags::real) && x.info(info_flags::real)) {
-		if (x.info(info_flags::positive))
+	if (y.is_real() && x.is_real()) {
+		if (x.is_positive())
 			return atan(y/x);
 
 		if (x.info(info_flags::negative)) {
-			if (y.info(info_flags::positive))
+			if (y.is_positive())
 				return atan(y/x)+Pi;
 			if (y.info(info_flags::negative))
 				return atan(y/x)-Pi;
@@ -1517,10 +1517,10 @@ static ex acot_eval(const ex & x)
 		if (x.is_zero())
 			return _ex1_2*Pi;
 
-		if (x.is_equal(_ex1))
+		if (x.is_one())
 			return _ex1_4*Pi;
 
-		if (x.is_equal(_ex_1))
+		if (x.is_minus_one())
 			return _ex_1_4*Pi;
 
 		if (x.is_equal(I) || x.is_equal(-I))
@@ -1574,7 +1574,10 @@ REGISTER_FUNCTION(acot, eval_func(acot_eval).
 static ex asec_evalf(const ex & x, PyObject* parent)
 {
 	if (is_exactly_a<numeric>(x))
-		return acos(ex_to<numeric>(x).inverse());
+        {
+                const numeric& num = ex_to<numeric>(x);
+                return acos(num.inverse());
+        }
 
 	return asec(x).hold();
 }
@@ -1582,7 +1585,7 @@ static ex asec_evalf(const ex & x, PyObject* parent)
 static ex asec_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
-                numeric num = ex_to<numeric>(x);
+                const numeric& num = ex_to<numeric>(x);
                 if (num.is_zero())
                         return NaN;
                 if (num.is_equal(*_num1_p))
@@ -1590,7 +1593,7 @@ static ex asec_eval(const ex & x)
                 if (num.is_equal(*_num_1_p))
                         return Pi;
                 if (num.info(info_flags::inexact))
-                        return acos(num.inverse());
+                        return asec_evalf(x, nullptr);
 	}
 
 	if (x.info(info_flags::infinity)) {
@@ -1638,8 +1641,10 @@ REGISTER_FUNCTION(asec, eval_func(asec_eval).
 // Needed because there is no Python RR equivalent
 static ex acsc_evalf(const ex & x, PyObject* parent)
 {
-	if (is_exactly_a<numeric>(x))
-		return asin(ex_to<numeric>(x).inverse());
+	if (is_exactly_a<numeric>(x)) {
+                const numeric& num = ex_to<numeric>(x);
+		return asin(num.inverse());
+        }
 
 	return acsc(x).hold();
 }
@@ -1647,7 +1652,7 @@ static ex acsc_evalf(const ex & x, PyObject* parent)
 static ex acsc_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
-                numeric num = ex_to<numeric>(x);
+                const numeric& num = ex_to<numeric>(x);
                 if (num.is_zero())
                         return NaN;
                 if (num.is_equal(*_num1_p))
