@@ -50,7 +50,6 @@ namespace GiNaC {
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(mul, expairseq,
   print_func<print_context>(&mul::do_print).
   print_func<print_latex>(&mul::do_print_latex).
-  print_func<print_csrc>(&mul::do_print_csrc).
   print_func<print_tree>(&mul::do_print_tree).
   print_func<print_python_repr>(&mul::do_print_python_repr))
 
@@ -134,7 +133,7 @@ DEFAULT_ARCHIVING(mul)
 // functions overriding virtual functions from base classes
 //////////
 
-void mul::print_overall_coeff(const ex coeff_ex, const print_context & c,
+void mul::print_overall_coeff(const ex& coeff_ex, const print_context & c,
 		const char *mul_sym, bool latex) const
 {
         if (not is_exactly_a<numeric>(coeff_ex))
@@ -156,9 +155,7 @@ void mul::print_overall_coeff(const ex coeff_ex, const print_context & c,
 		//(coeffstr.find('/') != std::string::npos) ||
 		//(coeffstr.find('*') != std::string::npos) ||
 		//(coeffstr.find('^') != std::string::npos));
-	if (num_coeff.is_integer()
-                and num_coeff.is_minus_one()
-                and not num_coeff.is_parent_pos_char())
+	if (num_coeff.is_minus_one())
 		c.s<<"-";
 	else if (parenthesis && coeffstr[0] == '-') {
 		// We want to move the '-' out of the parenthesis if it is
@@ -278,9 +275,7 @@ void mul::do_print_rat_func(const print_context & c, unsigned level,
 		if (latex_tags) {
 			const numeric& numer = overall_coeff.numer();
 			bool negate = false;
-			if (numer.is_integer()
-                            and numer.is_minus_one()
-                            and not numer.is_parent_pos_char()) {
+			if (numer.is_minus_one()) {
 				c.s<<"-";
 				negate = true;
 				//print_numer = coeff_numer.mul(*_num_1_p);
@@ -313,8 +308,8 @@ void mul::do_print_rat_func(const print_context & c, unsigned level,
 				}
 			}
 			c.s << "}{";
-			ex denom = overall_coeff.denom();
-			if (denom.is_equal(_ex1)) {
+			numeric denom = overall_coeff.denom();
+			if (denom.is_one()) {
 				mul(neg_powers).eval().print(c);
 			} else {
 				mul(denom, mul(neg_powers).eval()).hold().print(c);
@@ -322,7 +317,7 @@ void mul::do_print_rat_func(const print_context & c, unsigned level,
 			c.s << "}";
 		} else {
 			print_overall_coeff(overall_coeff, c,
-					others.size() == 0 ? "" : sep,
+					others.empty() ? "" : sep,
 					latex_tags);
 			if (others.empty()
                             and (overall_coeff.is_one()
@@ -382,63 +377,6 @@ void mul::do_print_rat_func(const print_context & c, unsigned level,
 
 }
 
-void mul::do_print_csrc(const print_csrc & c, unsigned level) const
-{
-	if (precedence() <= level)
-		c.s << "(";
-
-	if (not overall_coeff.is_one()) {
-		if (overall_coeff.is_minus_one()) {
-			c.s << "-";
-		}
-		else {
-			overall_coeff.print(c, precedence());
-			c.s << "*";
-		}
-	}
-
-	// Print arguments, separated by "*" or "/"
-	auto it = seq.begin(), itend = seq.end();
-	while (it != itend) {
-
-		// If the first argument is a negative integer power, it gets printed as "1.0/<expr>"
-		bool needclosingparenthesis = false;
-		if (it == seq.begin() && it->coeff.info(info_flags::negint)) {
-			if (is_a<print_csrc_cl_N>(c)) {
-				c.s << "recip(";
-				needclosingparenthesis = true;
-			} else
-				c.s << "1.0/";
-		}
-
-		// If the exponent is 1 or -1, it is left out
-		if (it->coeff.is_integer_pmone()) {
-			it->rest.print(c, precedence());
-		}
-		else if (it->coeff.info(info_flags::negint))
-			// Outer parens around ex needed for broken GCC parser:
-			(ex(power(it->rest, -ex_to<numeric>(it->coeff)))).print(c, level);
-		else
-			// Outer parens around ex needed for broken GCC parser:
-			(ex(power(it->rest, ex_to<numeric>(it->coeff)))).print(c, level);
-
-		if (needclosingparenthesis)
-			c.s << ")";
-
-		// Separator is "/" for negative integer powers, "*" otherwise
-		++it;
-		if (it != itend) {
-			if (it->coeff.info(info_flags::negint))
-				c.s << "/";
-			else
-				c.s << "*";
-		}
-	}
-
-	if (precedence() <= level)
-		c.s << ")";
-}
-
 void mul::do_print_python_repr(const print_python_repr & c, unsigned level) const
 {
 	c.s << class_name() << '(';
@@ -452,112 +390,126 @@ void mul::do_print_python_repr(const print_python_repr & c, unsigned level) cons
 
 bool mul::info(unsigned inf) const
 {
-	switch (inf) {
-		case info_flags::polynomial:
-		case info_flags::integer_polynomial:
-		case info_flags::cinteger_polynomial:
-		case info_flags::rational_polynomial:
-		case info_flags::real:
-		case info_flags::rational:
-		case info_flags::integer:
-		case info_flags::crational:
-		case info_flags::cinteger:
-                case info_flags::nonzero:
-		case info_flags::even:
-		case info_flags::crational_polynomial:
-		case info_flags::rational_function: {
-                        if (not overall_coeff.info(info_flags::real))
+        switch (inf) {
+        case info_flags::nonzero:
+                return is_positive()
+                or info(info_flags::negative);
+        case info_flags::polynomial:
+        case info_flags::integer_polynomial:
+        case info_flags::cinteger_polynomial:
+        case info_flags::rational_polynomial:
+        case info_flags::real:
+        case info_flags::rational:
+        case info_flags::integer:
+        case info_flags::crational:
+        case info_flags::cinteger:
+        case info_flags::crational_polynomial:
+        case info_flags::rational_function: {
+                if (not overall_coeff.is_real())
+                        return false;
+                for (const auto &elem : seq)
+                        if (!(recombine_pair_to_ex(elem).info(inf)))
                                 return false;
-                        for (const auto & elem : seq)
-				if (!(recombine_pair_to_ex(elem).info(inf)))
-					return false;
-			if (overall_coeff.is_one() && inf == info_flags::even)
-				return true;
-			return overall_coeff.info(inf);
-		}
-		case info_flags::inexact:
-		case info_flags::algebraic: {
-                        if (overall_coeff.info(inf))
+                if (overall_coeff.is_one() && inf == info_flags::even)
+                        return true;
+                return overall_coeff.info(inf);
+        }
+        case info_flags::even: {
+                bool even_seen = false;
+                for (const auto &elem : seq) {
+                        const ex &e = recombine_pair_to_ex(elem);
+                        if (not e.is_integer())
+                                return false;
+                        if (e.info(info_flags::even))
+                                even_seen = true;
+                }
+                return even_seen or overall_coeff.is_even();
+        }
+        case info_flags::inexact:
+        case info_flags::algebraic: {
+                if (overall_coeff.info(inf))
+                        return true;
+                for (const auto &elem : seq)
+                        if ((recombine_pair_to_ex(elem).info(inf)))
                                 return true;
-                        for (const auto & elem : seq)
-				if ((recombine_pair_to_ex(elem).info(inf)))
-					return true;
-			return false;
-		}
-		case info_flags::positive:
-		case info_flags::negative: {
-                        if (not overall_coeff.info(info_flags::real))
+                return false;
+        }
+        case info_flags::positive:
+        case info_flags::negative: {
+                if (not overall_coeff.is_real())
+                        return false;
+                bool pos = true;
+                for (const auto &elem : seq) {
+                        const ex &factor = recombine_pair_to_ex(elem);
+                        if (factor.is_positive())
+                                continue;
+                        if (factor.info(info_flags::negative))
+                                pos = !pos;
+                        else
                                 return false;
-			bool pos = true;
-                        for (const auto & elem : seq) {
-				const ex& factor = recombine_pair_to_ex(elem);
-				if (factor.info(info_flags::positive))
-					continue;
-				else if (factor.info(info_flags::negative))
-					pos = !pos;
-				else
-					return false;
-			}
-			if (overall_coeff.info(info_flags::negative))
-				pos = !pos;
-			return (inf ==info_flags::positive? pos : !pos);
-		}
-		case info_flags::nonnegative: {
-                        if (not overall_coeff.info(info_flags::real))
+                }
+                if (overall_coeff.info(info_flags::negative))
+                        pos = !pos;
+                return (inf == info_flags::positive ? pos : !pos);
+        }
+        case info_flags::nonnegative: {
+                if (not overall_coeff.is_real())
+                        return false;
+                bool pos = true;
+                for (const auto &elem : seq) {
+                        const ex &factor = recombine_pair_to_ex(elem);
+                        if (factor.info(info_flags::nonnegative)
+                            or factor.is_positive())
+                                continue;
+                        if (factor.info(info_flags::negative))
+                                pos = !pos;
+                        else
                                 return false;
-			bool pos = true;
-                        for (const auto & elem : seq) {
-				const ex& factor = recombine_pair_to_ex(elem);
-				if (factor.info(info_flags::nonnegative) || factor.info(info_flags::positive))
-					continue;
-				else if (factor.info(info_flags::negative))
-					pos = !pos;
-				else
-					return false;
-			}
-			return (overall_coeff.info(info_flags::negative)? !pos : pos);
-		}
-		case info_flags::posint:
-		case info_flags::negint: {
-                        if (not overall_coeff.info(info_flags::real))
+                }
+                return (overall_coeff.info(info_flags::negative) ? !pos : pos);
+        }
+        case info_flags::posint:
+        case info_flags::negint: {
+                if (not overall_coeff.is_real())
+                        return false;
+                bool pos = true;
+                for (const auto &elem : seq) {
+                        const ex &factor = recombine_pair_to_ex(elem);
+                        if (factor.info(info_flags::posint))
+                                continue;
+                        if (factor.info(info_flags::negint))
+                                pos = !pos;
+                        else
                                 return false;
-			bool pos = true;
-                        for (const auto & elem : seq) {
-				const ex& factor = recombine_pair_to_ex(elem);
-				if (factor.info(info_flags::posint))
-					continue;
-				else if (factor.info(info_flags::negint))
-					pos = !pos;
-				else
-					return false;
-			}
-			if (overall_coeff.info(info_flags::negint))
-				pos = !pos;
-			else if (!overall_coeff.info(info_flags::posint))
-				return false;
-			return (inf ==info_flags::posint? pos : !pos); 
-		}
-		case info_flags::nonnegint: {
-                        if (not overall_coeff.info(info_flags::real))
+                }
+                if (overall_coeff.info(info_flags::negint))
+                        pos = !pos;
+                else if (!overall_coeff.info(info_flags::posint))
+                        return false;
+                return (inf == info_flags::posint ? pos : !pos);
+        }
+        case info_flags::nonnegint: {
+                if (not overall_coeff.is_real())
+                        return false;
+                bool pos = true;
+                for (const auto &elem : seq) {
+                        const ex &factor = recombine_pair_to_ex(elem);
+                        if (factor.info(info_flags::nonnegint)
+                            or factor.info(info_flags::posint))
+                                continue;
+                        if (factor.info(info_flags::negint))
+                                pos = !pos;
+                        else
                                 return false;
-			bool pos = true;
-                        for (const auto & elem : seq) {
-				const ex& factor = recombine_pair_to_ex(elem);
-				if (factor.info(info_flags::nonnegint) || factor.info(info_flags::posint))
-					continue;
-				else if (factor.info(info_flags::negint))
-					pos = !pos;
-				else
-					return false;
-			}
-			if (overall_coeff.info(info_flags::negint))
-				pos = !pos;
-			else if (!overall_coeff.info(info_flags::posint))
-				return false;
-			return pos; 
-		}
-	}
-	return inherited::info(inf);
+                }
+                if (overall_coeff.info(info_flags::negint))
+                        pos = !pos;
+                else if (!overall_coeff.info(info_flags::posint))
+                        return false;
+                return pos;
+        }
+        }
+        return inherited::info(inf);
 }
 
 bool mul::is_polynomial(const ex & var) const
@@ -575,13 +527,20 @@ numeric mul::degree(const ex & s) const
 {
 	// Sum up degrees of factors
 	numeric deg_sum(0);
-        for (const auto & elem : seq)
-		if (ex_to<numeric>(elem.coeff).is_real())
-			deg_sum += recombine_pair_to_ex(elem).degree(s);
-		else {
-			if (elem.rest.has(s))
-				throw std::runtime_error("mul::degree() undefined degree because of non-integer exponent");
-		}
+        for (const auto & elem : seq) {
+		if (is_exactly_a<numeric>(elem.coeff)) {
+                        const numeric& n = ex_to<numeric>(elem.coeff);
+                        if (n.is_real()) {
+			        deg_sum += n * elem.rest.degree(s);
+                        }
+                        else {
+                                if (elem.rest.has(s))
+                                        throw std::runtime_error("mul::degree() undefined degree because of complex exponent");
+		        }
+                }
+                else
+		        throw std::runtime_error("mul::degree() undefined degree because of non-integer exponent");
+        }
 	return deg_sum;
 }
 
@@ -589,13 +548,20 @@ numeric mul::ldegree(const ex & s) const
 {
 	// Sum up degrees of factors
 	numeric deg_sum(0);
-        for (const auto & elem : seq)
-		if (ex_to<numeric>(elem.coeff).is_real())
-			deg_sum += recombine_pair_to_ex(elem).ldegree(s);
-		else {
-			if (elem.rest.has(s))
-				throw std::runtime_error("mul::ldegree() undefined degree because of non-integer exponent");
-		}
+        for (const auto & elem : seq) {
+		if (is_exactly_a<numeric>(elem.coeff)) {
+                        const numeric& n = ex_to<numeric>(elem.coeff);
+                        if (n.is_real()) {
+			        deg_sum += n * elem.rest.ldegree(s);
+                        }
+                        else {
+                                if (elem.rest.has(s))
+                                        throw std::runtime_error("mul::ldegree() undefined degree because of complex exponent");
+		        }
+                }
+                else
+		        throw std::runtime_error("mul::ldegree() undefined degree because of non-integer exponent");
+        }
 	return deg_sum;
 }
 
@@ -609,7 +575,7 @@ ex mul::coeff(const ex & s, const ex & n) const
 		// if a non-zero power of s is found, the resulting product will be 0
                 for (const auto & elem : seq)
 			coeffseq.push_back(recombine_pair_to_ex(elem).coeff(s,n));
-		coeffseq.push_back(overall_coeff);
+		coeffseq.emplace_back(overall_coeff);
 		return (new mul(coeffseq))->setflag(status_flags::dynallocated);
 	}
 	
@@ -625,7 +591,7 @@ ex mul::coeff(const ex & s, const ex & n) const
 		}
 	}
 	if (coeff_found) {
-		coeffseq.push_back(overall_coeff);
+		coeffseq.emplace_back(overall_coeff);
 		return (new mul(coeffseq))->setflag(status_flags::dynallocated);
 	}
 	
@@ -663,7 +629,7 @@ ex mul::eval(int level) const
 	
 	if (level == 1 and is_evaluated()) {
 		GINAC_ASSERT(seq.size()>0);
-		GINAC_ASSERT(seq.size()>1 || !overall_coeff.is_integer_one());
+		GINAC_ASSERT(seq.size()>1 || !overall_coeff.is_one());
 		return *this;
 	}
 
@@ -674,12 +640,21 @@ ex mul::eval(int level) const
 		           setflag(status_flags::dynallocated);
 	}
 	
-	// handle infinity and handle exp(a)*exp(b) -> exp(a+b) and
+	// handle infinity, simple way
+        if (std::any_of(seq.cbegin(), seq.cend(),
+                        [](expair p)
+                                { return is_exactly_a<infinity>(p.rest); })) {
+                infinity result = infinity::from_sign(1);
+                result *= overall_coeff;
+                for (const auto& p : seq)
+		        result *= recombine_pair_to_ex(p);
+                return result;
+        }
+
+        // handle exp(a)*exp(b) -> exp(a+b) and
 	unsigned exp_count = 0;
 	for (auto i = seq.begin(); i != seq.end(); i++) {
 		const numeric& num_coeff = ex_to<numeric>(i->coeff);
-		if (unlikely(is_exactly_a<infinity>(i->rest)))
-			return eval_infinity(i);
 		if (unlikely(is_ex_the_function(i->rest, exp) and
 			     num_coeff.is_integer())) {
 			exp_count++;
@@ -694,18 +669,20 @@ ex mul::eval(int level) const
 	if (overall_coeff.is_zero()) {
 		// *(...,x;0) -> 0
 		return overall_coeff;
-	} else if (seq_size==0) {
+	}
+        if (seq_size == 0) {
 		// *(;c) -> c
 		return overall_coeff;
-	} else if (seq_size==1
-                   and overall_coeff.is_one()
-		   and not overall_coeff.is_parent_pos_char()) {
+	}
+        if (seq_size == 1
+            and overall_coeff.is_one()) {
 		// *(x;1) -> x
 		// except in positive characteristic: 1*(x+2) = x in F_2
 		return recombine_pair_to_ex(*(seq.begin()));
-	} else if ((seq_size==1) &&
-	           is_exactly_a<add>((*seq.begin()).rest) &&
-	           ex_to<numeric>((*seq.begin()).coeff).is_one()) {
+	}
+        if (seq_size == 1
+	     and is_exactly_a<add>((*seq.begin()).rest)
+	     and ex_to<numeric>((*seq.begin()).coeff).is_one()) {
 		// *(+(x,y,...);c) -> +(*(x,c),*(y,c),...) (c numeric(), no powers of +())
 		const add & addref = ex_to<add>((*seq.begin()).rest);
 		epvector distrseq;
@@ -716,14 +693,14 @@ ex mul::eval(int level) const
 		                addref.overall_coeff.mul(overall_coeff))
 		       )->setflag(status_flags::dynallocated | status_flags::evaluated);
                 const add & result = ex_to<add>(x);
-                if (result.seq.size()==0)
+                if (result.seq.empty())
                         return result.overall_coeff;
-                else
-                        return x;
-	} else if ((seq_size >= 2) && ((flags & status_flags::expanded) == 0u)) {
+                return x;
+	}
+        if (seq_size >= 2
+            and ((flags & status_flags::expanded) == 0u)) {
 		// Strip the content and the unit part from each term. Thus
-		// things like (-x+a)*(3*x-3*a) automagically turn into - 3*(x-a)2
-
+		// things like (-x+a)*(3*x-3*a) automagically turn into - 3*(x-a)^2
 		auto last = seq.end();
 		auto i = seq.begin();
 		auto j = seq.begin();
@@ -731,7 +708,7 @@ ex mul::eval(int level) const
 		numeric oc = *_num1_p;
 		bool something_changed = false;
 		while (i!=last) {
-			if (likely(! (is_exactly_a<add>(i->rest) && i->coeff.is_integer_one()))) {
+			if (likely(! (is_exactly_a<add>(i->rest) && i->coeff.is_one()))) {
 				// power::eval has such a rule, no need to handle powers here
 				++i;
 				continue;
@@ -774,15 +751,10 @@ ex mul::eval(int level) const
 				++j;
 			}
 
-			if (! unit_normal) {
-				if(unlikely(c.get_parent_char() == 2) && c.is_one()) {
-					c = *_num_1_p;
-				} else {
-					c = c.mul(*_num_1_p);
-				}
-			}
+                        if (!unit_normal)
+                                c = c.mul(*_num_1_p);
 
-			oc = oc.mul(c);
+                        oc = oc.mul(c);
 
 			// divide add by the number in place to save at least 2 .eval() calls
 			const add& addref = ex_to<add>(i->rest);
@@ -794,7 +766,7 @@ ex mul::eval(int level) const
 			for (auto & elem : primitive->seq)
 				elem.coeff = ex_to<numeric>(elem.coeff).div_dyn(c);
 
-			s.push_back(expair(*primitive, _ex1));
+			s.emplace_back(*primitive, _ex1);
 
 			++i;
 			++j;
@@ -837,29 +809,12 @@ ex mul::eval_exponentials() const
 	if (is_exactly_a<numeric>(new_exp))
 		oc = oc.mul(ex_to<numeric>(new_exp));
 	else
-		s.push_back(expair(new_exp, _ex1));
+		s.emplace_back(new_exp, _ex1);
 
 	mul * result = new mul(s, overall_coeff.mul(oc));
 	return result->setflag(status_flags::dynallocated);
 }
 
-
-ex mul::eval_infinity(epvector::const_iterator infinity_iter) const
-{
-	GINAC_ASSERT(is_exactly_a<infinity>(infinity_iter->rest));
-	GINAC_ASSERT(infinity_iter->coeff.is_equal(_ex1));
-	infinity result(ex_to<numeric>(
-                ex_to<infinity>(infinity_iter->rest).get_direction()));
-        result *= overall_coeff;
-
-	for (auto i = seq.begin(); i != seq.end(); i++) {
-		if (i == infinity_iter) continue;
-		result *= recombine_pair_to_ex(*i);
-	}
-	return result;
-}
-
-
 ex mul::evalf(int level, PyObject* parent) const
 {
 	if (level==1)
@@ -920,7 +875,8 @@ bool tryfactsubs(const ex & origfactor, const ex & patternfactor, int & nummatch
 	int origexponent;
 	int origexpsign;
 
-	if (is_exactly_a<power>(origfactor) && origfactor.op(1).info(info_flags::integer)) {
+	if (is_exactly_a<power>(origfactor)
+            and origfactor.op(1).is_integer()) {
 		origbase = origfactor.op(0);
 		int expon = ex_to<numeric>(origfactor.op(1)).to_int();
 		origexponent = expon > 0 ? expon : -expon;
@@ -935,7 +891,8 @@ bool tryfactsubs(const ex & origfactor, const ex & patternfactor, int & nummatch
 	int patternexponent;
 	int patternexpsign;
 
-	if (is_exactly_a<power>(patternfactor) && patternfactor.op(1).info(info_flags::integer)) {
+	if (is_exactly_a<power>(patternfactor)
+            and patternfactor.op(1).is_integer()) {
 		patternbase = patternfactor.op(0);
 		int expon = ex_to<numeric>(patternfactor.op(1)).to_int();
 		patternexponent = expon > 0 ? expon : -expon;
@@ -989,8 +946,7 @@ bool algebraic_match_mul_with_mul(const mul &e, const ex &pat, lst &repls,
 				nummatches = newnummatches;
 				return true;
 			}
-			else
-				matched[i] = false;
+			matched[i] = false;
 		}
 	}
 
@@ -1114,7 +1070,7 @@ ex mul::derivative(const symbol & s) const
 		expair ep = split_ex_to_pair(power(i->rest, i->coeff - _ex1) *
 		                             i->rest.diff(s));
 		ep.swap(*i2);
-		addseq.push_back((new mul(mulseq,
+		addseq.emplace_back((new mul(mulseq,
                      overall_coeff.mul(ex_to<numeric>(i->coeff))))->setflag(status_flags::dynallocated));
 		ep.swap(*i2);
 		++i; ++i2;
@@ -1237,7 +1193,7 @@ expair mul::combine_pair_with_coeff_to_pair(const expair & p,
                 return expair(p.rest, p.coeff*c);
 	if (c.is_one())
 		return p;
-        if (p.coeff.is_integer_one())
+        if (p.coeff.is_one())
                 return expair(p.rest, c);
 
 	// to avoid duplication of power simplification rules,
@@ -1249,12 +1205,14 @@ expair mul::combine_pair_with_coeff_to_pair(const expair & p,
 	
 ex mul::recombine_pair_to_ex(const expair & p) const
 {
-        if (unlikely(is_exactly_a<infinity>(p.rest)))
-                return (new infinity(ex_to<numeric>(p.coeff)))->setflag(status_flags::evaluated|status_flags::dynallocated);
-	if (p.coeff.is_integer_one()) 
+        if (unlikely(is_exactly_a<infinity>(p.rest))) {
+                infinity res = ex_to<infinity>(p.rest);
+                res *= p.coeff;
+                return res;
+        }
+	if (p.coeff.is_one()) 
 		return p.rest;
-	else
-		return (new power(p.rest,p.coeff))->setflag(status_flags::dynallocated);
+	return (new power(p.rest,p.coeff))->setflag(status_flags::dynallocated);
 }
 
 bool mul::expair_needs_further_processing(epp it)
@@ -1272,7 +1230,7 @@ bool mul::expair_needs_further_processing(epp it)
 			*it = ep;
 			return true;
 		}
-		if (it->coeff.is_integer_one()) {
+		if (it->coeff.is_one()) {
 			// combined pair has coeff 1 and must be moved to the end
 			return true;
 		}
@@ -1303,7 +1261,7 @@ bool mul::can_make_flat(const expair & p) const
 	GINAC_ASSERT(is_exactly_a<numeric>(p.coeff));
 
         // (x*y)^c == x^c*y^c  if c ∈ ℤ
-        return p.coeff.info(info_flags::integer);
+        return p.coeff.is_integer();
 }
 
 bool mul::can_be_further_expanded(const ex & e)
@@ -1325,7 +1283,8 @@ ex mul::expand(unsigned options) const
 	// trivial case: expanding the monomial (~ 30% of all calls)
         bool all_intsym = true;
         for (const auto & elem : seq) 
-                if (not is_exactly_a<symbol>(elem.rest) or not elem.coeff.info(info_flags::integer))
+                if (not is_exactly_a<symbol>(elem.rest)
+                    or not elem.coeff.is_integer())
                 {
                         all_intsym = false;
                         break;
@@ -1337,7 +1296,7 @@ ex mul::expand(unsigned options) const
 
 	// First, expand the children
 	std::unique_ptr<epvector> expanded_seqp = expandchildren(options);
-	const epvector & expanded_seq = (expanded_seqp.get() != nullptr ? *expanded_seqp : seq);
+	const epvector & expanded_seq = (expanded_seqp != nullptr ? *expanded_seqp : seq);
 
 	// Now, look for all the factors that are sums and multiply each one out
 	// with the next one that is found while collecting the factors which are
@@ -1349,7 +1308,7 @@ ex mul::expand(unsigned options) const
 
 	for (const auto & elem : expanded_seq) {
 		if (is_exactly_a<add>(elem.rest) &&
-			(elem.coeff.is_integer_one())) {
+			(elem.coeff.is_one())) {
 			if (is_exactly_a<add>(last_expanded)) {
 				// Expand a product of two sums, aggressive version.
 				// Caring for the overall coefficients in separate loops can
@@ -1386,8 +1345,8 @@ ex mul::expand(unsigned options) const
 						distrseq.insert(distrseq.end(),add2begin,add2end);
 					else
                                                 for (const auto & elem2 : add2.seq)
-							distrseq.push_back(expair(elem2.rest,
-                                                                ex_to<numeric>(elem2.coeff).mul_dyn(add1.overall_coeff)));
+							distrseq.emplace_back(elem2.rest,
+                                                                ex_to<numeric>(elem2.coeff).mul_dyn(add1.overall_coeff));
 				}
 
 				// Multiply add1 with the overall coefficient of add2 and append it to distrseq:
@@ -1396,7 +1355,7 @@ ex mul::expand(unsigned options) const
 						distrseq.insert(distrseq.end(),add1begin,add1end);
 					else
                                                 for (const auto & elem1 : add1.seq)
-							distrseq.push_back(expair(elem1.rest, ex_to<numeric>(elem1.coeff).mul_dyn(add2.overall_coeff)));
+							distrseq.emplace_back(elem1.rest, ex_to<numeric>(elem1.coeff).mul_dyn(add2.overall_coeff));
 				}
 
 				// Compute the new overall coefficient and put it together:
@@ -1418,14 +1377,14 @@ ex mul::expand(unsigned options) const
 						if (is_exactly_a<numeric>(rest)) {
 							oc += ex_to<numeric>(rest).mul(ex_to<numeric>(elem1.coeff).mul(ex_to<numeric>(elem2.coeff)));
 						} else {
-							distrseq2.push_back(expair(rest, ex_to<numeric>(elem1.coeff).mul_dyn(ex_to<numeric>(elem2.coeff))));
+							distrseq2.emplace_back(rest, ex_to<numeric>(elem1.coeff).mul_dyn(ex_to<numeric>(elem2.coeff)));
 						}
 					}
 					tmp_accu += (new add(distrseq2, oc))->setflag(status_flags::dynallocated);
 				} 
 				last_expanded = tmp_accu;
 			} else {
-				if (!last_expanded.is_integer_one())
+				if (!last_expanded.is_one())
 					non_adds.push_back(split_ex_to_pair(last_expanded));
 				last_expanded = elem.rest;
 			}
@@ -1460,14 +1419,14 @@ ex mul::expand(unsigned options) const
 	}
 
 	non_adds.push_back(split_ex_to_pair(last_expanded));
-	ex result = (new mul(non_adds, overall_coeff))->setflag(status_flags::dynallocated);
+	ex result = (new mul(non_adds, overall_coeff))->
+                setflag(status_flags::dynallocated);
 	if (can_be_further_expanded(result)) {
 		return result.expand();
-	} else {
-		if (options == 0)
-			ex_to<basic>(result).setflag(status_flags::expanded);
-		return result;
-	}
+	} 
+        if (options == 0)
+                ex_to<basic>(result).setflag(status_flags::expanded);
+        return result;
 }
 
 const epvector & mul::get_sorted_seq() const
