@@ -132,7 +132,7 @@ typedef std::pair<ex, ex> ex2;
 typedef std::pair<ex, bool> exbool;
 
 struct ex2_less {
-	bool operator() (const ex2 &p, const ex2 &q) const 
+	bool operator() (const ex2 &p, const ex2 &q) const
 	{
 		int cmp = p.first.compare(q.first);
 		return ((cmp<0) || (!(cmp>0) && p.second.compare(q.second)<0));
@@ -196,33 +196,147 @@ numeric mul::max_coefficient() const
 
 bool ex::is_linear(const symbol& x, ex& a, ex& b) const
 {
-        if (degree(x) > 1)
+        expand();
+        if (not has_symbol(*this, x)) {
+                a = *this;
+                b = _ex0;
+                return true;
+        }
+        if (this->is_equal(x)) {
+                a = _ex0;
+                b = _ex1;
+                return true;
+        }
+        if (is_exactly_a<mul>(*this)) {
+                if (has_symbol(*this/x, x))
+                        return false;
+                a = _ex0;
+                b = *this/x;
+                return true;
+        }
+        if (not is_exactly_a<add>(*this))
                 return false;
-        b = coeff(x, 1);
-        if (has_symbol(b,x))
-                return false;
-        a = ((*this) - b*x).expand();
-        if (has_symbol(a,x))
+        const add& A = ex_to<add>(*this);
+        exvector cterms, xterms;
+        for (unsigned i=0; i<A.nops(); ++i)
+                if (has_symbol(A.op(i), x))
+                        xterms.push_back(A.op(i));
+                else
+                        cterms.push_back(A.op(i));
+        ex xt = (add(xterms) / x).normal();
+        if (has_symbol(xt, x))
                 return false;
+        a = add(cterms);
+        b = xt;
         return true;
 }
 
 bool ex::is_quadratic(const symbol& x, ex& a, ex& b, ex& c) const
 {
-        if (degree(x) > 2)
-                return false;
-        c = coeff(x, 2);
-        if (has_symbol(c,x))
-                return false;
-        b = coeff(x, 1);
-        if (has_symbol(b,x))
-                return false;
+        expand();
+        expairvec coeffs;
+        coefficients(x, coeffs);
+        b = c = _ex0;
+        for (const auto& p : coeffs) {
+                const ex& d = p.second;
+                if (d.is_equal(_ex2)) {
+                        c = p.first;
+                        if (has_symbol(c,x))
+                                return false;
+                }
+                else if (d.is_equal(_ex1)) {
+                        b = p.first;
+                        if (has_symbol(b,x))
+                                return false;
+                }
+                else if (not d.is_equal(_ex0))
+                        return false;
+        }
         a = ((*this) - c*power(x,2) - b*x).expand();
         if (has_symbol(a,x))
                 return false;
         return true;
 }
 
+bool ex::is_binomial(const symbol& x, ex& a, ex& j, ex& b, ex& n) const
+{
+        expand();
+        if (is_linear(x, a, b)) {
+                j = _ex0;
+                if (b.is_zero())
+                        n = _ex0;
+                else
+                        n = _ex1;
+                return true;
+        }
+        if (is_exactly_a<power>(*this)) {
+                const power& p = ex_to<power>(*this);
+                if (has_symbol(p.op(1), x)
+                    or not p.op(0).is_equal(x))
+                        return false;
+                a = _ex1;
+                j = p.op(1);
+                b = _ex0;
+                n = _ex0;
+                return true;
+        }
+        if (is_exactly_a<mul>(*this)) {
+                const mul& m = ex_to<mul>(*this);
+                ex cprod = _ex1;
+                j = _ex0;
+                b = _ex0;
+                n = _ex0;
+                for (unsigned i=0; i<m.nops(); ++i) {
+                        const ex& factor = m.op(i);
+                        if (not has_symbol(factor, x))
+                                cprod *= factor;
+                        else if (is_exactly_a<power>(factor)) {
+                                const power& pow = ex_to<power>(factor);
+                                if (has_symbol(pow.op(1), x)
+                                    or not pow.op(0).is_equal(x))
+                                        return false;
+                                j = pow.op(1);
+                        }
+                        else if (factor.is_equal(x))
+                                j = _ex1;
+                        else
+                                return false;
+                }
+                a = cprod;
+                return true;
+        }
+        if (not is_exactly_a<add>(*this))
+                return false;
+        const add& A = ex_to<add>(*this);
+        exvector cterms, xterms;
+        for (unsigned i=0; i<A.nops(); ++i)
+                if (has_symbol(A.op(i), x))
+                        xterms.push_back(A.op(i));
+                else
+                        cterms.push_back(A.op(i));
+        if (xterms.size() > 2
+            or (xterms.size() == 2
+                and cterms.size() > 0))
+                return false;
+
+        ex ta, tj, tb, tn;
+        bool r = xterms[0].is_binomial(x, ta, tj, tb, tn);
+        if (not r)
+                return false;
+        a = ta;
+        j = tj;
+        if (xterms.size() < 2) {
+                b = add(cterms);
+                n = _ex0;
+                return true;
+        }
+        r = xterms[1].is_binomial(x, ta, tj, tb, tn);
+        if (not r)
+                return false;
+        b = ta;
+        n = tj;
+        return true;
+}
 
 /** Apply symmetric modular homomorphism to an expanded multivariate
  *  polynomial.  This function was usually used internally by heur_gcd().
@@ -250,7 +364,7 @@ ex add::smod(const numeric &xi) const
 		GINAC_ASSERT(!is_exactly_a<numeric>(it->rest));
 		numeric num_coeff = GiNaC::smod(ex_to<numeric>(it->coeff), xi);
 		if (!num_coeff.is_zero())
-			newseq.push_back(expair(it->rest, num_coeff));
+			newseq.emplace_back(it->rest, num_coeff);
 		it++;
 	}
 	numeric num_coeff = GiNaC::smod(overall_coeff, xi);
@@ -353,16 +467,14 @@ ex basic::normal(exmap & repl, exmap & rev_lookup, int level, unsigned options)
 {
 	if (nops() == 0)
 		return (new lst(replace_with_symbol(*this, repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
-	else {
-		if (level == 1)
-			return (new lst(replace_with_symbol(*this, repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
-		else if (level == -max_recursion_level)
-			throw(std::runtime_error("max recursion level reached"));
-		else {
-			normal_map_function map_normal(level - 1);
-			return (new lst(replace_with_symbol(map(map_normal), repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
-		}
-	}
+        if (level == 1)
+                return (new lst(replace_with_symbol(*this, repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
+        if (level == -max_recursion_level)
+                throw(std::runtime_error("max recursion level reached"));
+        else {
+                normal_map_function map_normal(level - 1);
+                return (new lst(replace_with_symbol(map(map_normal), repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
+        }
 }
 
 
@@ -411,7 +523,7 @@ static ex frac_cancel(const ex &n, const ex &d)
 //std::clog << "frac_cancel num = " << num << ", den = " << den << std::endl;
 
 	// Handle trivial case where denominator is 1
-	if (den.is_equal(_ex1))
+	if (den.is_one())
 		return (new lst(num, den))->setflag(status_flags::dynallocated);
 
 	// Handle special cases where numerator or denominator is 0
@@ -430,7 +542,7 @@ static ex frac_cancel(const ex &n, const ex &d)
 
 	// Cancel GCD from numerator and denominator
 	ex cnum, cden;
-	if (not gcdpoly(num, den, &cnum, &cden, false).is_integer_one()) {
+	if (not gcdpoly(num, den, &cnum, &cden, false).is_one()) {
 		num = cnum;
 		den = cden;
 	}
@@ -466,7 +578,7 @@ ex function::normal(exmap & repl, exmap & rev_lookup, int level, unsigned option
         }
         if (level == 1)
                 return (new lst(replace_with_symbol(*this, repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
-        else if (level == -max_recursion_level)
+        if (level == -max_recursion_level)
                 throw(std::runtime_error("max recursion level reached"));
         else {
                 normal_map_function map_normal(level - 1);
@@ -481,7 +593,7 @@ ex add::normal(exmap & repl, exmap & rev_lookup, int level, unsigned options) co
 {
 	if (level == 1)
 		return (new lst(replace_with_symbol(*this, repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
-	else if (level == -max_recursion_level)
+	if (level == -max_recursion_level)
 		throw(std::runtime_error("max recursion level reached"));
 
 	// Normalize children and split each one into numerator and denominator
@@ -541,7 +653,7 @@ ex mul::normal(exmap & repl, exmap & rev_lookup, int level, unsigned options) co
 {
 	if (level == 1)
 		return (new lst(replace_with_symbol(*this, repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
-	else if (level == -max_recursion_level)
+	if (level == -max_recursion_level)
 		throw(std::runtime_error("max recursion level reached"));
 
 	// Normalize children, separate into numerator and denominator
@@ -572,7 +684,7 @@ ex power::normal(exmap & repl, exmap & rev_lookup, int level, unsigned options)
 {
 	if (level == 1)
 		return (new lst(replace_with_symbol(*this, repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
-	else if (level == -max_recursion_level)
+	if (level == -max_recursion_level)
 		throw(std::runtime_error("max recursion level reached"));
 
 	// Normalize basis and exponent (exponent gets reassembled)
@@ -580,43 +692,55 @@ ex power::normal(exmap & repl, exmap & rev_lookup, int level, unsigned options)
 	ex n_exponent = ex_to<basic>(exponent).normal(repl, rev_lookup, level-1);
 	n_exponent = n_exponent.op(0) / n_exponent.op(1);
 
-	if (n_exponent.info(info_flags::integer)) {
-
-		if (n_exponent.info(info_flags::positive)) {
+	if (n_exponent.is_integer()) {
 
+		if (n_exponent.is_positive()) {
 			// (a/b)^n -> {a^n, b^n}
-			return (new lst(power(n_basis.op(0), n_exponent), power(n_basis.op(1), n_exponent)))->setflag(status_flags::dynallocated);
+			return (new lst(power(n_basis.op(0), n_exponent),
+                                              power(n_basis.op(1), n_exponent)))
+                                ->setflag(status_flags::dynallocated);
 
-		} else if (n_exponent.info(info_flags::negative)) {
-
-			// (a/b)^-n -> {b^n, a^n}
-			return (new lst(power(n_basis.op(1), -n_exponent), power(n_basis.op(0), -n_exponent)))->setflag(status_flags::dynallocated);
 		}
-
+                if (n_exponent.info(info_flags::negative)) {
+			// (a/b)^-n -> {b^n, a^n}
+			return (new lst(power(n_basis.op(1), -n_exponent),
+                                              power(n_basis.op(0), -n_exponent)))
+                                ->setflag(status_flags::dynallocated);
+		}
 	} else {
 
-		if (n_exponent.info(info_flags::positive)) {
-
+		if (n_exponent.is_positive()) {
 			// (a/b)^x -> {sym((a/b)^x), 1}
-			return (new lst(replace_with_symbol(power(n_basis.op(0) / n_basis.op(1), n_exponent), repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
+			return (new lst(replace_with_symbol(power(n_basis.op(0) / n_basis.op(1),
+                                                                n_exponent),
+                                                            repl, rev_lookup),
+                                                _ex1))
+                                ->setflag(status_flags::dynallocated);
+		}
+                if (n_exponent.info(info_flags::negative)) {
 
-		} else if (n_exponent.info(info_flags::negative)) {
-
-			if (n_basis.op(1).is_equal(_ex1)) {
+			if (n_basis.op(1).is_one()) {
 
 				// a^-x -> {1, sym(a^x)}
-				return (new lst(_ex1, replace_with_symbol(power(n_basis.op(0), -n_exponent), repl, rev_lookup)))->setflag(status_flags::dynallocated);
-
-			} else {
-
-				// (a/b)^-x -> {sym((b/a)^x), 1}
-				return (new lst(replace_with_symbol(power(n_basis.op(1) / n_basis.op(0), -n_exponent), repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
+				return (new lst(_ex1,
+                                                replace_with_symbol(power(n_basis.op(0), -n_exponent),
+                                                        repl, rev_lookup)))
+                                        ->setflag(status_flags::dynallocated);
 			}
+
+                // (a/b)^-x -> {sym((b/a)^x), 1}
+                return (new lst(replace_with_symbol(power(n_basis.op(1) / n_basis.op(0), -n_exponent),
+                                                repl, rev_lookup), _ex1))
+                        ->setflag(status_flags::dynallocated);
+			
 		}
 	}
 
 	// (a/b)^x -> {sym((a/b)^x, 1}
-	return (new lst(replace_with_symbol(power(n_basis.op(0) / n_basis.op(1), n_exponent), repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
+	return (new lst(replace_with_symbol(power(n_basis.op(0) / n_basis.op(1),
+                                                n_exponent),
+                                        repl, rev_lookup), _ex1))
+                ->setflag(status_flags::dynallocated);
 }
 
 
@@ -630,7 +754,7 @@ ex pseries::normal(exmap & repl, exmap & rev_lookup, int level, unsigned options
 	while (i != end) {
 		ex restexp = i->rest.normal();
 		if (!restexp.is_zero())
-			newseq.push_back(expair(restexp, i->coeff));
+			newseq.emplace_back(restexp, i->coeff);
 		++i;
 	}
 	ex n = pseries(relational(var,point), newseq);
@@ -670,8 +794,8 @@ ex ex::normal(int level, bool noexpand_combined, bool noexpand_numer) const
         // Convert {numerator, denominator} form back to fraction
         if ((options & normal_options::no_expand_fraction_numer) == 0u)
                 return e.op(0).expand() / e.op(1);
-        else
-        	return e.op(0) / e.op(1);
+
+        return e.op(0) / e.op(1);
 }
 
 /** Get numerator of an expression. If the expression is not of the normal
@@ -867,10 +991,10 @@ ex numeric::to_polynomial(exmap & repl) const
  *  powers by temporary symbols. */
 ex power::to_rational(exmap & repl) const
 {
-	if (exponent.info(info_flags::integer))
+	if (exponent.is_integer())
 		return power(basis.to_rational(repl), exponent);
-	else
-		return replace_with_symbol(*this, repl);
+
+	return replace_with_symbol(*this, repl);
 }
 
 /** Implementation of ex::to_polynomial() for powers. It replaces non-posint
@@ -879,19 +1003,19 @@ ex power::to_polynomial(exmap & repl) const
 {
 	if (exponent.info(info_flags::posint))
 		return power(basis.to_rational(repl), exponent);
-	else if (exponent.info(info_flags::negint))
+	if (exponent.info(info_flags::negint))
 	{
 		ex basis_pref = collect_common_factors(basis);
-		if (is_exactly_a<mul>(basis_pref) || is_exactly_a<power>(basis_pref)) {
+		if (is_exactly_a<mul>(basis_pref)
+                    or is_exactly_a<power>(basis_pref)) {
 			// (A*B)^n will be automagically transformed to A^n*B^n
 			ex t = power(basis_pref, exponent);
 			return t.to_polynomial(repl);
 		}
-		else
-			return power(replace_with_symbol(power(basis, _ex_1), repl), -exponent);
-	} 
-	else
-		return replace_with_symbol(*this, repl);
+		return power(replace_with_symbol(power(basis, _ex_1), repl),
+                                -exponent);
+	}
+	return replace_with_symbol(*this, repl);
 }
 
 
@@ -908,8 +1032,8 @@ ex expairseq::to_rational(exmap & repl) const
 	ex oc = overall_coeff.to_rational(repl);
 	if (oc.info(info_flags::numeric))
 		return thisexpairseq(s, overall_coeff);
-	else
-		s.push_back(expair(oc, _ex1));
+
+	s.emplace_back(oc, _ex1);
 	return thisexpairseq(s, default_overall_coeff());
 }
 
@@ -926,8 +1050,8 @@ ex expairseq::to_polynomial(exmap & repl) const
 	ex oc = overall_coeff.to_polynomial(repl);
 	if (oc.info(info_flags::numeric))
 		return thisexpairseq(s, overall_coeff);
-	else
-		s.push_back(expair(oc, _ex1));
+	
+		s.emplace_back(oc, _ex1);
 	return thisexpairseq(s, default_overall_coeff());
 }
 
@@ -961,7 +1085,7 @@ static ex find_common_factor(const ex & e, ex & factor, exmap & repl)
 			terms.push_back(x);
 		}
 
-		if (gc.is_equal(_ex1))
+		if (gc.is_one())
 			return e;
 #ifdef PYNAC_HAVE_LIBGIAC
                 else {
@@ -1001,8 +1125,8 @@ static ex find_common_factor(const ex & e, ex & factor, exmap & repl)
 term_done:	;
 		}
 		return (new add(terms))->setflag(status_flags::dynallocated);
-
-	} else if (is_exactly_a<mul>(e)) {
+	}
+        if (is_exactly_a<mul>(e)) {
 
 		size_t num = e.nops();
 		exvector v; v.reserve(num);
@@ -1011,21 +1135,19 @@ term_done:	;
 			v.push_back(find_common_factor(e.op(i), factor, repl));
 
 		return (new mul(v))->setflag(status_flags::dynallocated);
-
-	} else if (is_exactly_a<power>(e)) {
+	}
+        if (is_exactly_a<power>(e)) {
 		const ex e_exp(e.op(1));
-		if (e_exp.info(info_flags::integer)) {
+		if (e_exp.is_integer()) {
 			ex eb = e.op(0).to_polynomial(repl);
 			ex factor_local(_ex1);
 			ex pre_res = find_common_factor(eb, factor_local, repl);
 			factor *= power(factor_local, e_exp);
 			return power(pre_res, e_exp);
-			
-		} else
-			return e.to_polynomial(repl);
-
-	} else
-		return e;
+		}
+		return e.to_polynomial(repl);
+        }
+        return e;
 }
 
 
@@ -1040,7 +1162,7 @@ ex collect_common_factors(const ex & e)
 		ex r = find_common_factor(e, factor, repl);
 		return factor.subs(repl, subs_options::no_pattern) * r.subs(repl, subs_options::no_pattern);
 
-	} else
+	}
 		return e;
 }
 
@@ -1078,7 +1200,7 @@ bool factor(const ex& the_ex, ex& res_ex)
                         res_ex = mul(ev);
                 return mchanged;
         }
-        else if (is_exactly_a<power>(the_ex)) {
+        if (is_exactly_a<power>(the_ex)) {
                 const power& p = ex_to<power>(the_ex);
                 ex r;
                 bool pchanged = factor(p.op(0), r);
