@@ -95,7 +95,7 @@ pseries::pseries(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
 		ex coef;
 		n.find_ex_by_loc(loc++, rest, sym_lst);
 		n.find_ex_by_loc(loc++, coef, sym_lst);
-		seq.push_back(expair(rest, coef));
+		seq.emplace_back(rest, coef);
 	}
 
 	n.find_ex("var", var, sym_lst);
@@ -113,8 +113,6 @@ void pseries::archive(archive_node &n) const
 	n.add_ex("point", point);
 }
 
-DEFAULT_UNARCHIVE(pseries)
-
 //////////
 // functions overriding virtual functions from base classes
 //////////
@@ -140,7 +138,7 @@ void pseries::print_series(const print_context & c, const char *openbrace, const
 
 			// print 'rest', i.e. the expansion coefficient
 			if (i->rest.info(info_flags::numeric) &&
-				i->rest.info(info_flags::positive)) {
+				i->rest.is_positive()) {
 				i->rest.print(c);
 			} else {
 				c.s << openbrace << '(';
@@ -285,13 +283,13 @@ numeric pseries::degree(const ex &s) const
 {
 	if (var.is_equal(s)) {
 		// Return last exponent
-		if (seq.size() != 0u)
+		if (!seq.empty())
 			return ex_to<numeric>((seq.end()-1)->coeff).to_int();
-		else
+		
 			return 0;
 	} else {
                 
-		if (seq.size() == 0)
+		if (seq.empty())
 			return 0;
 		int max_pow = std::numeric_limits<int>::min();
                 for (const auto & elem : seq) {
@@ -312,12 +310,12 @@ numeric pseries::ldegree(const ex &s) const
 {
 	if (var.is_equal(s)) {
 		// Return first exponent
-		if (seq.size() != 0u)
+		if (!seq.empty())
 			return ex_to<numeric>((seq.begin())->coeff);
-		else
+		
 			return 0;
 	} else {
-		if (seq.size() == 0)
+		if (seq.empty())
 			return 0;
 		int min_pow = std::numeric_limits<int>::max();
                 for (const auto & elem : seq) {
@@ -345,27 +343,27 @@ ex pseries::coeff(const ex &s, const ex & n) const
 		// Binary search in sequence for given power
                 if (not is_exactly_a<numeric>(n))
                         throw std::runtime_error("can't happen in pseries::coeff");
-		numeric looking_for = ex_to<numeric>(n);
+		const numeric& looking_for = ex_to<numeric>(n);
 		int lo = 0, hi = seq.size() - 1;
 		while (lo <= hi) {
 			int mid = (lo + hi) / 2;
 			GINAC_ASSERT(is_exactly_a<numeric>(seq[mid].coeff));
 			int cmp = ex_to<numeric>(seq[mid].coeff).compare(looking_for);
-			switch (cmp) {
-				case -1:
-					lo = mid + 1;
-					break;
-				case 0:
-					return seq[mid].rest;
-				case 1:
-					hi = mid - 1;
-					break;
-				default:
-					throw(std::logic_error("pseries::coeff: compare() didn't return -1, 0 or 1"));
-			}
-		}
+                        switch (cmp) {
+                        case -1:
+                                lo = mid + 1;
+                                break;
+                        case 0:
+                                return seq[mid].rest;
+                        case 1:
+                                hi = mid - 1;
+                                break;
+                        default:
+                                throw(std::logic_error("pseries::coeff: compare() didn't return -1, 0 or 1"));
+                        }
+                }
 		return _ex0;
-	} else
+	} 
 		return convert_to_poly().coeff(s, n);
 }
 
@@ -388,7 +386,7 @@ ex pseries::eval(int level) const
 	epvector new_seq;
 	new_seq.reserve(seq.size());
         for (const auto & elem : seq)
-		new_seq.push_back(expair(elem.rest.eval(level-1), elem.coeff));
+		new_seq.emplace_back(elem.rest.eval(level-1), elem.coeff);
 	return (new pseries(relational(var,point), new_seq))->setflag(status_flags::dynallocated | status_flags::evaluated);
 }
 
@@ -405,14 +403,14 @@ ex pseries::evalf(int level, PyObject* parent) const
 	epvector new_seq;
 	new_seq.reserve(seq.size());
         for (const auto & elem : seq)
-		new_seq.push_back(expair(elem.rest.evalf(level-1, parent),
-					elem.coeff));
+		new_seq.emplace_back(elem.rest.evalf(level-1, parent),
+					elem.coeff);
 	return (new pseries(relational(var,point), new_seq))->setflag(status_flags::dynallocated | status_flags::evaluated);
 }
 
 ex pseries::conjugate() const
 {
-	if(!var.info(info_flags::real))
+	if (not var.is_real())
 		return conjugate_function(*this).hold();
 
 	std::unique_ptr<epvector> newseq(conjugateepvector(seq));
@@ -428,7 +426,7 @@ ex pseries::conjugate() const
 
 ex pseries::real_part() const
 {
-	if(!var.info(info_flags::real))
+	if (not var.is_real())
 		return real_part_function(*this).hold();
 	ex newpoint = point.real_part();
 	if(newpoint != point)
@@ -437,13 +435,13 @@ ex pseries::real_part() const
 	epvector v;
 	v.reserve(seq.size());
 	for(const auto & elem : seq)
-		v.push_back(expair((elem.rest).real_part(), elem.coeff));
+		v.emplace_back((elem.rest).real_part(), elem.coeff);
 	return (new pseries(var==point, v))->setflag(status_flags::dynallocated);
 }
 
 ex pseries::imag_part() const
 {
-	if(!var.info(info_flags::real))
+	if (not var.is_real())
 		return imag_part_function(*this).hold();
 	ex newpoint = point.real_part();
 	if(newpoint != point)
@@ -452,7 +450,7 @@ ex pseries::imag_part() const
 	epvector v;
 	v.reserve(seq.size());
 	for(const auto & elem : seq)
-		v.push_back(expair((elem.rest).imag_part(), elem.coeff));
+		v.emplace_back((elem.rest).imag_part(), elem.coeff);
 	return (new pseries(var==point, v))->setflag(status_flags::dynallocated);
 }
 
@@ -469,7 +467,7 @@ ex pseries::subs(const exmap & m, unsigned options) const
 	epvector newseq;
 	newseq.reserve(seq.size());
         for (const auto & elem : seq)
-		newseq.push_back(expair(elem.rest.subs(m, options), elem.coeff));
+		newseq.emplace_back(elem.rest.subs(m, options), elem.coeff);
 	return (new pseries(relational(var,point.subs(m, options)), newseq))->setflag(status_flags::dynallocated);
 }
 
@@ -481,7 +479,7 @@ ex pseries::expand(unsigned options) const
         for (const auto & elem : seq) {
 		ex restexp = elem.rest.expand();
 		if (!restexp.is_zero())
-			newseq.push_back(expair(restexp, elem.coeff));
+			newseq.emplace_back(restexp, elem.coeff);
 	}
 	return (new pseries(relational(var,point), newseq))
 	        ->setflag(status_flags::dynallocated | (options == 0 ? status_flags::expanded : 0));
@@ -498,11 +496,11 @@ ex pseries::derivative(const symbol & s) const
 		// FIXME: coeff might depend on var
                 for (const auto & elem : seq) {
 			if (is_order_function(elem.rest)) {
-				new_seq.push_back(expair(elem.rest, elem.coeff - 1));
+				new_seq.emplace_back(elem.rest, elem.coeff - 1);
 			} else {
 				const ex& c = elem.rest * elem.coeff;
 				if (!c.is_zero())
-					new_seq.push_back(expair(c, elem.coeff - 1));
+					new_seq.emplace_back(c, elem.coeff - 1);
 			}
 		}
 
@@ -514,7 +512,7 @@ ex pseries::derivative(const symbol & s) const
 			} else {
 				const ex& c = elem.rest.diff(s);
 				if (!c.is_zero())
-					new_seq.push_back(expair(c, elem.coeff));
+					new_seq.emplace_back(c, elem.coeff);
 			}
 		}
 	}
@@ -569,7 +567,7 @@ ex basic::series(const relational & r, int order, unsigned options) const
 
 	// default for order-values that make no sense for Taylor expansion
 	if ((order <= 0) && this->has(s)) {
-		seq.push_back(expair(Order(_ex1), order));
+		seq.emplace_back(Order(_ex1), order);
 		return pseries(r, seq);
 	}
 
@@ -578,7 +576,7 @@ ex basic::series(const relational & r, int order, unsigned options) const
 	ex deriv = *this;
 	const ex co = deriv.subs(r, subs_options::no_pattern);
 	if (not co.is_zero()) {
-		seq.push_back(expair(co, _ex0));
+		seq.emplace_back(co, _ex0);
 	}
 
 	int n;
@@ -593,13 +591,13 @@ ex basic::series(const relational & r, int order, unsigned options) const
 
 		const ex coef = deriv.subs(r, subs_options::no_pattern);
 		if (!coef.is_zero())
-			seq.push_back(expair(fac.inverse() * coef, n));
+			seq.emplace_back(fac.inverse() * coef, n);
 	}
 	
 	// Higher-order terms, if present
 	deriv = deriv.diff(s);
         if (!deriv.expand(expand_options::expand_only_numerators).is_zero())
-		seq.push_back(expair(Order(_ex1), n));
+		seq.emplace_back(Order(_ex1), n);
 	return pseries(r, seq);
 }
 
@@ -608,8 +606,8 @@ ex numeric::series(const relational & r, int order, unsigned options) const
 {
 	epvector seq;
         if (not is_zero())
-                seq.push_back(expair(*this, _ex0));
-        seq.push_back(expair(Order(_ex1), numeric(order)));
+                seq.emplace_back(*this, _ex0);
+        seq.emplace_back(Order(_ex1), numeric(order));
 	return pseries(r, seq);
 }
 
@@ -623,13 +621,13 @@ ex symbol::series(const relational & r, int order, unsigned options) const
 
 	if (this->is_equal_same_type(ex_to<symbol>(r.lhs()))) {
 		if (order > 0 && !point.is_zero())
-			seq.push_back(expair(point, _ex0));
+			seq.emplace_back(point, _ex0);
 		if (order > 1)
-			seq.push_back(expair(_ex1, _ex1));
+			seq.emplace_back(_ex1, _ex1);
 		else
-			seq.push_back(expair(Order(_ex1), numeric(order)));
+			seq.emplace_back(Order(_ex1), numeric(order));
 	} else
-		seq.push_back(expair(*this, _ex0));
+		seq.emplace_back(*this, _ex0);
 	return pseries(r, seq);
 }
 
@@ -645,7 +643,7 @@ ex pseries::add_series(const pseries &other) const
 	// results in an empty (constant) series 
 	if (!is_compatible_to(other)) {
 		epvector nul;
-		nul.push_back(expair(Order(_ex1), _ex0));
+		nul.emplace_back(Order(_ex1), _ex0);
 		return pseries(relational(var,point), nul);
 	}
 	
@@ -664,7 +662,7 @@ ex pseries::add_series(const pseries &other) const
 				++b;
 			}
 			break;
-		} else
+		} 
 			pow_a = ex_to<numeric>((*a).coeff).to_int();
 		
 		// If b is empty, fill up with elements from a and stop
@@ -674,7 +672,7 @@ ex pseries::add_series(const pseries &other) const
 				++a;
 			}
 			break;
-		} else
+		} 
 			pow_b = ex_to<numeric>((*b).coeff).to_int();
 		
 		// a and b are non-empty, compare powers
@@ -693,15 +691,15 @@ ex pseries::add_series(const pseries &other) const
 		} else {
 			// Add coefficient of a and b
 			if (is_order_function((*a).rest) || is_order_function((*b).rest)) {
-				new_seq.push_back(expair(Order(_ex1), (*a).coeff));
+				new_seq.emplace_back(Order(_ex1), (*a).coeff);
 				break;  // Order term ends the sequence
-			} else {
+			} 
 				ex sum = (*a).rest + (*b).rest;
 				if (!(sum.is_zero()))
-					new_seq.push_back(expair(sum, numeric(pow_a)));
+					new_seq.emplace_back(sum, numeric(pow_a));
 				++a;
 				++b;
-			}
+			
 		}
 	}
 	return pseries(relational(var,point), new_seq);
@@ -747,7 +745,7 @@ ex pseries::mul_const(const numeric &other) const
 	
         for (const auto & elem : seq) {
 		if (!is_order_function(elem.rest))
-			new_seq.push_back(expair(elem.rest * other, elem.coeff));
+			new_seq.emplace_back(elem.rest * other, elem.coeff);
 		else
 			new_seq.push_back(elem);
 	}
@@ -766,7 +764,7 @@ ex pseries::mul_series(const pseries &other) const
 	// results in an empty (constant) series 
 	if (!is_compatible_to(other)) {
 		epvector nul;
-		nul.push_back(expair(Order(_ex1), _ex0));
+		nul.emplace_back(Order(_ex1), _ex0);
 		return pseries(relational(var,point), nul);
 	}
 
@@ -816,10 +814,10 @@ ex pseries::mul_series(const pseries &other) const
 				co += it1->second * it2->second;
 		}
 		if (!co.is_zero())
-			new_seq.push_back(expair(co, numeric(cdeg)));
+			new_seq.emplace_back(co, numeric(cdeg));
 	}
 	if (higher_order_c < std::numeric_limits<int>::max())
-		new_seq.push_back(expair(Order(_ex1), numeric(higher_order_c)));
+		new_seq.emplace_back(Order(_ex1), numeric(higher_order_c));
 	return pseries(relational(var, point), new_seq);
 }
 
@@ -846,7 +844,7 @@ ex mul::series(const relational & r, int order, unsigned options) const
 		ex expon = elem.coeff;
 		int factor = 1;
 		ex buf;
-		if (expon.info(info_flags::integer)) {
+		if (expon.is_integer()) {
 			buf = elem.rest;
 			factor = ex_to<numeric>(expon).to_int();
 		} else {
@@ -892,7 +890,7 @@ ex mul::series(const relational & r, int order, unsigned options) const
 			ex expon = elem.coeff;
 			int factor = 1;
 			ex buf;
-			if (expon.info(info_flags::integer)) {
+			if (expon.is_integer()) {
 				buf = elem.rest;
 				factor = ex_to<numeric>(expon).to_int();
 			} else {
@@ -915,7 +913,7 @@ ex mul::series(const relational & r, int order, unsigned options) const
 
 	if (degsum >= order) {
 		epvector epv;
-		epv.push_back(expair(Order(_ex1), order));
+		epv.emplace_back(Order(_ex1), order);
 		return (new pseries(r, epv))->setflag(status_flags::dynallocated);
 	}
 
@@ -984,7 +982,7 @@ ex pseries::power_const(const numeric &p, int deg) const
 	if (numcoeff <= 0) {
 		epvector epv;
 		epv.reserve(1);
-		epv.push_back(expair(Order(_ex1), deg));
+		epv.emplace_back(Order(_ex1), deg);
 		return (new pseries(relational(var,point), epv))
 		       ->setflag(status_flags::dynallocated);
 	}
@@ -996,15 +994,15 @@ ex pseries::power_const(const numeric &p, int deg) const
 	// Compute coefficients of the powered series
 	exvector co;
 	co.reserve(numcoeff);
-	co.push_back(power(coeff(var, ldeg), p));
+	co.emplace_back(power(coeff(var, ldeg), p));
 	for (int i=1; i<numcoeff; ++i) {
 		ex sum = _ex0;
 		for (int j=1; j<=i; ++j) {
 			ex c = coeff(var, j + ldeg);
 			if (is_order_function(c)) {
-				co.push_back(Order(_ex1));
+				co.emplace_back(Order(_ex1));
 				break;
-			} else
+			} 
 				sum += (p * j - (i - j)) * co[i - j] * c;
 		}
 		co.push_back(sum / coeff(var, ldeg) / i);
@@ -1015,14 +1013,14 @@ ex pseries::power_const(const numeric &p, int deg) const
 	bool higher_order = false;
 	for (int i=0; i<numcoeff; ++i) {
 		if (!co[i].is_zero())
-			new_seq.push_back(expair(co[i], p * ldeg + i));
+			new_seq.emplace_back(co[i], p * ldeg + i);
 		if (is_order_function(co[i])) {
 			higher_order = true;
 			break;
 		}
 	}
 	if (!higher_order)
-		new_seq.push_back(expair(Order(_ex1), p * ldeg + numcoeff));
+		new_seq.emplace_back(Order(_ex1), p * ldeg + numcoeff);
 
 	return pseries(relational(var,point), new_seq);
 }
@@ -1100,9 +1098,9 @@ ex power::series(const relational & r, int order, unsigned options) const
 		epvector new_seq;
 		if (is_exactly_a<numeric>(exponent)
                     and ex_to<numeric>(exponent).to_int() < order)
-			new_seq.push_back(expair(_ex1, exponent));
+			new_seq.emplace_back(_ex1, exponent);
 		else
-			new_seq.push_back(expair(Order(_ex1), exponent));
+			new_seq.emplace_back(Order(_ex1), exponent);
 		return pseries(r, new_seq);
 	}
 
@@ -1137,7 +1135,7 @@ ex power::series(const relational & r, int order, unsigned options) const
 		result = ex_to<pseries>(e).power_const(numexp, order);
 	} catch (pole_error) {
 		epvector ser;
-		ser.push_back(expair(Order(_ex1), order));
+		ser.emplace_back(Order(_ex1), order);
 		result = pseries(r, ser);
 	}
 
@@ -1155,18 +1153,18 @@ ex pseries::series(const relational & r, int order, unsigned options) const
 	if (var.is_equal(s) && point.is_equal(p)) {
 		if (order > degree(s))
 			return *this;
-		else {
+		
 			epvector new_seq;
                         for (const auto & elem : seq) {
 				int o = ex_to<numeric>(elem.coeff).to_int();
 				if (o >= order) {
-					new_seq.push_back(expair(Order(_ex1), o));
+					new_seq.emplace_back(Order(_ex1), o);
 					break;
 				}
 				new_seq.push_back(elem);
 			}
 			return pseries(r, new_seq);
-		}
+		
 	} else
 		return convert_to_poly().series(r, order, options);
 }
