@@ -54,7 +54,7 @@ inline int compare_pointers(const T * a, const T * b)
 	// but std::less is.
 	if (std::less<const T *>()(a, b))
 		return -1;
-	else if (std::less<const T *>()(b, a))
+	if (std::less<const T *>()(b, a))
 		return 1;
 	return 0;
 }
@@ -343,7 +343,6 @@ extern const ex _ex_1_3;
 extern const numeric *_num_1_4_p;
 extern const ex _ex_1_4;
 extern const numeric *_num0_p;
-extern const basic *_num0_bp;
 extern const ex _ex0;
 extern const numeric *_num1_4_p;
 extern const ex _ex1_4;
@@ -395,8 +394,6 @@ extern const numeric *_num25_p;
 extern const ex _ex25;
 extern const numeric *_num26_p;
 extern const ex _ex26;
-extern const numeric *_num26_p;
-extern const ex _ex26;
 extern const numeric *_num27_p;
 extern const ex _ex27;
 extern const numeric *_num28_p;
@@ -422,15 +419,8 @@ extern const ex _ex144;
 #define DEFAULT_CTOR(classname) \
 classname::classname() : inherited(&classname::tinfo_static) { setflag(status_flags::evaluated | status_flags::expanded); }
 
-#define DEFAULT_UNARCHIVE(classname) \
-ex classname::unarchive(const archive_node &n, lst &sym_lst) \
-{ \
-	return (new classname(n, sym_lst))->setflag(status_flags::dynallocated); \
-}
-
 #define DEFAULT_ARCHIVING(classname) \
 classname::classname(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst) {} \
-DEFAULT_UNARCHIVE(classname) \
 void classname::archive(archive_node &n) const \
 { \
 	inherited::archive(n); \
@@ -515,6 +505,67 @@ range_t<T> range(T b, T e)
     return range_t<T>(b, e);
 }
 
+// adapted from http://en.cppreference.com/w/cpp/algorithm/next_permutation
+template<class BidirIt>
+int next_permutation_pos(BidirIt first, BidirIt last)
+{
+    if (first == last) return -1;
+    BidirIt i = last;
+    if (first == --i) return -1;
+ 
+    while (true) {
+        BidirIt i1, i2;
+ 
+        i1 = i;
+        if (*--i < *i1) {
+            i2 = last;
+            while (!(*i < *--i2))
+                ;
+            std::iter_swap(i, i2);
+            std::reverse(i1, last);
+            return i1 - first;
+        }
+        if (i == first) {
+            std::reverse(first, last);
+            return -1;
+        }
+    }
+}
+
+// choice of r out of n, given a container [0,...,n]
+template <class C>
+int next_combination(std::vector<C>& vec, size_t r, size_t n)
+{
+        if (vec.empty()) {
+                for (size_t i=0; i<r; ++i)
+                        vec.push_back(i);
+                return n>1 and r<n and r>0;
+        }
+        if (n<2 or r==n)
+                return false;
+        auto first = vec.begin(), last = vec.end();
+        if ((*first) != n - r) {
+                auto mt = last;
+                while (*(--mt) == n-(last-mt));
+                (*mt)++;
+                while (++mt != last) *mt = *(mt-1)+1;
+                return true;
+        }
+        return false;
+}
+
+template<class C, class H>
+bool subset_of(const std::unordered_set<C,H>& s1,
+               const std::unordered_set<C,H>& s2)
+{
+        if (s1.size() > s2.size())
+                return false;
+        for (const auto& elem : s1)
+                if (s2.find(elem) == s2.end())
+                        return false;
+        return true;
+}
+
 } // namespace GiNaC
 
 
